#include <kernel/include.h>
#include <kernel/schedule.h>
#include <kernel/list.h>
#include <drivers/serial/serial.h>
#include <drivers/lcd/define.h>
#include <drivers/rtc/rtc.h>
#include <drivers/key/key.h>
#include <drivers/fnd/fnd.h>
//#include <drivers/textlcd/textlcd.h>


#include "mainimg.h"
#include "main0.h"
#include "img0.h"
#include "img0_0_36x64.h"
#include "img0_0_o.h"
#include "img0_1_88x72.h"
#include "img0_1_o.h"
#include "img0_2_56x48.h"
#include "img0_2_o.h"
#include "img1.h"
#include "loading_140x28.h"
#define ON	1
#define OFF	0



char task_key_queue[4] = {0xf, 0xf, 0xf, 0xf};
int selecting = 0;


void task_key();
void task_home_network();
void task_coffee_maker();
void task_statistics();


int C_Entry()
{
	rtc_init(7, 0, 0);

	Arch_Init();
	Kern_Init();
	Kern_IOSubBuild();

	Kern_ThreadCreate(1,"key task", &task_key, 1, 0);
	Kern_ThreadCreate(2,"home network", &task_home_network, 2, 0);
	Kern_ThreadCreate(3,"coffee maker", &task_coffee_maker, 3, 0);

	lcd_putpic(loading, 250, 226, 140, 28, 0x10);
	lcd_putpic(mainimg, 0, 0, 640, 480, 0x11);

	OSTimer0_IRQ_Init();
	Kern_Sched();

	while(1) {;}

	return 0;
}

void display_menu(void)
{
	SerialOutputString("## menu ## ");
	switch (selecting) {
		case 2:
			SerialOutputString("<home network>   coffee maker \n");
			break;
		case 3:
			SerialOutputString(" home network   <coffee maker>\n");
			break;
		default:
			SerialOutputString(" home network    coffee maker \n");
			break;
	}
	SerialOutputString("(8):[menu] (4):[home network]    (0):[coffee_maker]\n");
	SerialOutputString("(9):[task] (5):light on/off      (1):premium\n");
	SerialOutputString("(A):-      (6):audio on/off      (2):normal\n");
	SerialOutputString("(B):-      (7):television on/off (3):cancle\n");
	SerialOutputString("push key button!\n");
}

char key_wait(int task_no)
{
	char keyin;

	while (task_key_queue[task_no] == 0xf) ;
	keyin = task_key_queue[task_no];
	task_key_queue[task_no] = 0xf;

	return keyin;
}

void task_key(void)
{
	char keyin;

	for (;;) {
		keyin = key_read();
		switch (keyin) {
			case 0x0:
				lcd_putpic(loading, 250, 226, 140, 28, 0x00);
				lcd_putbmp(img1, 160, 120, 320, 240, 0x10);
				selecting = 3;
				display_menu();
				break;
			case 0x4:
				lcd_putpic(loading, 250, 226, 140, 28, 0x00);
				lcd_putbmp(img0, 160, 120, 320, 240, 0x10);
				selecting = 2;
				display_menu();
				break;
			case 0x8:
				lcd_putpic(loading, 250, 226, 140, 28, 0x00);
				lcd_putpic(main0, 160, 120, 320, 240, 0x10);
				selecting = 0;
				display_menu();
				break;
			case 0x9:
				lcd_putpic(loading, 250, 226, 140, 28, 0x00);
				lcd_putpic(main0, 160, 120, 320, 240, 0x10);
				Kern_ThreadCreate(4,"statistics", &task_statistics, 4, 0);
				selecting = 4;
				display_menu();
				break;
			default:
				task_key_queue[selecting] = keyin;
				break;
		}
	}
}

void task_home_network(void)
{
	char no;

	int toggle0, toggle1, toggle2;
	toggle0 = OFF;
	toggle1 = OFF;
	toggle2 = OFF;

	for (;;) {
		no = key_wait(2);

		switch (no) {
			case 0x5:	// light
				if (toggle0 == OFF) {
					SerialOutputString("light on!\n");
					lcd_putpic(img0_0_o, 160+120, 120+0, 36, 64, 0x00);
					toggle0 = ON;
				}
				else {
					SerialOutputString("light off!\n");
					lcd_putbmp(img0_0, 160+120, 120+0, 36, 64, 0x00);
					toggle0 = OFF;
				}
				break;
			case 0x6:	// audio
				if (toggle1 == OFF) {
					SerialOutputString("audio on!\n");
					lcd_putbmp(img0_1_o, 160+50, 120+168, 88, 72, 0x00);
					toggle1 = ON;
				}
				else {
					SerialOutputString("audio off!\n");
					lcd_putbmp(img0_1, 160+50, 120+168, 88, 72, 0x00);
					toggle1 = OFF;
				}
				break;
			case 0x7:	// television
				if (toggle2 == OFF) {
					SerialOutputString("television on!\n");
					lcd_putbmp(img0_2_o, 160+202, 120+98, 56, 48, 0x00);
					toggle2 = ON;
				}
				else {
					SerialOutputString("television off!\n");
					lcd_putbmp(img0_2, 160+202, 120+98, 56, 48, 0x00);
					toggle2 = OFF;
				}
				break;
			default:
				SerialOutputString("invaild key!\n");
				break;
		}
	}
}

void task_coffee_maker(void)
{
	char no;

	for (;;) {
		no = key_wait(3);

		switch (no) {
			case 0x1:
				SerialOutputString("making premium coffee! waiting for 30 second!\n");
				rtc_alarm_init(0, 0, 30);
				break;
			case 0x2:
				SerialOutputString("making normal coffee! waiting for 15 second!\n");
				rtc_alarm_init(0, 0, 15);
				break;
			case 0x3:
				SerialOutputString("coffee order cancle!\n");
				rtc_alarm_release();
				break;
			default:
				SerialOutputString("invaild key!\n");
				break;
		}
	}
}

void task_statistics(void)
{
	char no;
	tcb_t *p;

	lcd_setpos(0, 0);
	lcd_setfgbg(0xFFFF, 0xF000);
	lcd_putstring("** Current Process List **");

	for (;;) {
		no = key_wait(4);

		switch (no) {
			case 0xa:
				SerialOutputString("Statistics Task Terminate!!\n");
				return;
				break;
			default:
				break;
		}

		lcd_setpos(0, 2);
		ListForEachEntry(p, &rdyListHead_g.threadList, threadList) {
			//SerialOutputString(p->name);
			lcd_setfgbg(0xFFFF, 50);
			lcd_putstring("Thread id : ");
			lcd_puthex(p->tid);
			lcd_setfgbg(0xFFFF, 0xFFFF);
			lcd_putstring("\r\n");
			lcd_setfgbg(0xFFFF, 50);
			lcd_putstring("  name : ");
			lcd_putstring(p->name);
			lcd_setfgbg(0xFFFF, 0xFFFF);
			lcd_putstring("\r\n");
			lcd_putstring("  status : ");
			//lcd_puthex(p->status);
			lcd_putstring("running");
			lcd_setfgbg(0xFFFF, 0xFFFF);
			lcd_putstring("\r\n");
			lcd_putstring("\r\n");
		}
	}
}
