
.text
#include <arch/arch.h>

.extern Kern_Sched

.global _Exception_undef
.global _Exception_swi
.global _Exception_pabort
.global _Exception_dabort
.global _Exception_irq
.global _Exit_irq
.global _Exception_fiq
.global _nop_loop



//----------------- Exception handler
_Exception_undef:
	mov	r0, #'u'
	bl	serial_write
	b	_nop_loop
	movs	pc, lr

_Exception_swi:
	mov	r0, #'s'
	bl	serial_write
	b	_nop_loop
	movs	pc, lr

_Exception_pabort:
	mov	r0, #'p'
	bl	serial_write
	b	_nop_loop
	subs	pc, lr, #0x4

_Exception_dabort:
	mov	r0, #'d'
	bl	serial_write
	b	_nop_loop
	subs	pc, lr, #0x8

_Exception_irq:
	stmfd	sp!, {r0-r12}
	sub	lr, lr, #4
	stmfd	sp!, {lr}

;	// IRQ check
	ldr	r2, =0x04000000
	ldr	r1, =INT_BASE
	ldr	r0, [r1, #INT_ICIP]
	tst	r0, r2
	bne	OSTimer0_IRQ

	bl	Irq_Handler

;	// If not, come back routine
	ldmfd	sp!, {lr}
	ldmfd	sp!, {r0-r12}
	movs	pc, lr


_Exit_irq:
;	// Previous task's context push
	ldmfd	r12!, {r0}
	stmfd	sp!, {r0}
	stmfd	sp!, {lr}
	ldmfd	r12!, {r0-r12}
	stmfd	sp!, {r0-r12}

;	// I-bit clear
	mrs	r12, cpsr
	bic	r12, r12, #0x80
	msr	cpsr, r12
	stmfd	sp!, {r12}

	ldr	r0, =pNextThread_g
	ldr	r0, [r0]
	str	sp, [r0]

//	b	Kern_TickISR
	b	Kern_Sched


_Exception_fiq:
	mov	r0, #'f'
	bl	serial_write
	b	_nop_loop
	subs	pc, lr, #4




_nop_loop:
	nop
	b	_nop_loop
