
#include <arch/arch.h>
#include <arch/memmap.h>

.global _Init_board

.text

_Init_board:
//----------------- Initialize XScale
	// Enter SVC mode and disable all IRQ and FIQs
	mov	r0, #0x000000d3
	msr	cpsr_c, r0

	// Enable access to all coprocessors
	ldr	r0, =0x2001
	mcr 	p15, 0, r0, c15, c1, 0
	nop
	nop
	nop

	// Disable MMU, caches, write buffer
	ldr 	r0, =0x00000078
	mcr 	p15, 0, r0, c1, c0, 0
	nop
	nop
	nop

	ldr	r0, =0x00000000
	mcr	p15, 0, r0, c8, c7, 0		// flush TLB's
	mcr	p15, 0, r0, c7, c7, 0		// flush Caches
	mcr	p15, 0, r0, c7, c10, 4		// flush Write Buffer
	nop
	nop
	nop

	// Grant manager access to all domains
	mvn    r0, #0
	mcr    p15, 0, r0, c3, c0, 0


	// All interrupt level set IRQ & Mask off
	ldr	r1, =INT_BASE
	ldr	r2, =0x0
	str	r2, [r1, #INT_ICLR]
	str	r2, [r1, #INT_ICMR]


	// Initialize PXA250 Clock Register
	ldr	r1, =CLK_BASE		// base of clock registers

	ldr	r2, =0x00000040
	str	r2, [r1, #CLK_CKEN]
	ldr	r2, =0x00000002
	str	r2, [r1, #CLK_OSCC]
	ldr	r2, =0x00000241
	str	r2, [r1, #CLK_CCCR]

	ldr	r1, =TMR_BASE
	mov	r2, #0x0
	str	r2, [r1, #TMR_OSCR]
	ldr	r3, =0x300
wait_for_clock:
	ldr	r2, [r1, #TMR_OSCR]
	cmp	r3, r2
	ble	wait_for_clock


	// PXA250 Core Clock set Turbo Mode
	ldr	r0, =0
	mrc	p14, 0, r0, c6, c0, 0
	ldr	r0, =3
	mcr	p14, 0, r0, c6, c0, 0


//----------------- Initialize GPIO
	// Initialize FFUART
	ldr	r12, =FFUART_BASE

	ldr	r0, =0x00000000
	str	r0, [r12, #FFLCR]
	ldr	r0, =0x00000000
	str	r0, [r12, #FFIER]
	ldr	r0, =0x00000080
	str	r0, [r12, #FFLCR]
	ldr	r0, =0x00000018
	str	r0, [r12, #FFDLL]
	ldr	r0, =0x00000000
	str	r0, [r12, #FFDLH]
	ldr	r0, =0x00000000
	str	r0, [r12, #FFLCR]
	ldr	r0, =0x00000003
	str	r0, [r12, #FFLCR]
	ldr	r0, =0x00000001
	str	r0, [r12, #FFFCR]
	ldr	r0, =0x00000007
	str	r0, [r12, #FFFCR]
	ldr	r0, =0x00000000
	str	r0, [r12, #FFIER]
	ldr	r0, =0x00000000
	str	r0, [r12, #FFMCR]

	ldr	r0, =FFISR_VALUE
	str	r0, [r12, #FFISR]

	ldr	r12, =FFUART_BASE
	ldr	r0, =0x00000040
	str	r0, [r12, #FFIER]

	// Clear the rx fifo errors
	ldr	r0, =FFFCR_VALUE
	and	r0, r0, #0x00000002
	str	r0, [r12, #FFFCR]


	// First set the output values to a safe/disabled state before we change 
	// any GPIO's outputs
	// start by settings all of them high which is the safest for most signals

	ldr	r12, =GPIO_BASE

	ldr	r0, =GAFR0L_VALUE
	str	r0, [r12, #GAFR0_L]
	ldr	r0, =GAFR0U_VALUE
	str	r0, [r12, #GAFR0_U]

	ldr	r0, =GAFR1L_VALUE
	str	r0, [r12, #GAFR1_L]
	ldr	r0, =GAFR1U_VALUE
	str	r0, [r12, #GAFR1_U]

	ldr	r0, =GAFR2L_VALUE
	str	r0, [r12, #GAFR2_L]
	ldr	r0, =GAFR2U_VALUE
	str	r0, [r12, #GAFR2_U]

	ldr	r0, =GPSR0_VALUE
	str	r0, [r12, #GPSR0]
	ldr	r0, =GPSR1_VALUE
	str	r0, [r12, #GPSR1]
	ldr	r0, =GPSR2_VALUE
	str	r0, [r12, #GPSR2]

	ldr	r0, =GPCR0_VALUE
	str	r0, [r12, #GPCR0]
	ldr	r0, =GPCR1_VALUE
	str	r0, [r12, #GPCR1]
	ldr	r0, =GPCR2_VALUE
	str	r0, [r12, #GPCR2]

	ldr	r0, =GPDR0_VALUE
	str	r0, [r12, #GPDR0]
	ldr	r0, =GPDR1_VALUE
	str	r0, [r12, #GPDR1]
	ldr	r0, =GPDR2_VALUE
	str	r0, [r12, #GPDR2]


	// Clear the peripheral control registers bits
	ldr r12, =PSSR
	ldr r0, =(PSSR_RDH | PSSR_PH)
	str r0, [r12]


//----------------- Initialize Memory Controller
	// Initialize Static Memory Controller
	ldr	r1, =MSC_BASE		// Base of memory controller

	ldr	r2, =0x3f695af0		// CS0 : FLASH , CS1 : EPLD (not used)
    	str	r2, [r1, #MSC_MSC0]
	ldr	r3, [r1, #MSC_MSC0]

    	ldr	r2, =0x3ff97ffc		// CS2 : CS8900A , CS3 : FPGA board & IO board)
    	str	r2, [r1, #MSC_MSC1]
    	ldr	r3, [r1, #MSC_MSC1]

	ldr	r2, =0x3ff97ff9		// CS4 : USB1.0 HOST , CS5 : KEY DECODER
	str	r2, [r1, #MSC_MSC2]
	ldr	r3, [r1, #MSC_MSC2]


	// Initialize SDRAM Controller
	ldr	r2, =0x03ca4fff    		// Build MDREFR in a specific order
	str	r2, [r1, #MSC_MDREFR]

    	ldr	r2, =0x03ca4030			// REF Rate = (64MS/8192 Rows) * 100MHz /32 = 24
    	str	r2, [r1, #MSC_MDREFR]

	ldr	r2, =0x03cf6030			// Set K0RUN, K1RUN and K2RUN
	str	r2, [r1, #MSC_MDREFR]
		
	ldr	r2, =0x038f6030			// Clear Self Refresh
	str	r2, [r1, #MSC_MDREFR]
		
	ldr	r2, =0x038ff030			// Set E0PIN and E1PIN
	str	r2, [r1, #MSC_MDREFR]

	ldr	r2, =0x0ac80ac8			// Set CNFG, but don't enable just yet
						// CL=3, tRP=3clk, tRC=10clk, tRAS=7clk, tRCD=3clk
	str	r2, [r1, #MSC_MDCNFG]

	// Pause for ~200usec for SDRAM to init
	ldr	r3, =0x400
loop_delay:
	nop
	nop
	nop
	subs	r3, r3, #1
	bne	loop_delay
	nop
	nop
	nop


	// Force 9 cycles to SDRAM : raise CBR(refrash cycle)
	ldr     r2, =SDRAM_START
	str     r2, [r2]
	str     r2, [r2]
	str     r2, [r2]
	str     r2, [r2]
	str     r2, [r2]
	str     r2, [r2]
	str     r2, [r2]
	str     r2, [r2]
	str     r2, [r2]


	// Now enable SDRAM
	ldr	r1, =MSC_BASE
	ldr	r2, =0x0acb0acb
	str	r2, [r1, #MSC_MDCNFG]


	// And do an MRS
	ldr	r2, =0x00220022
	str	r2, [r1, #MSC_MDMRS]

//----------------- End init
	mov	pc, lr
