
#include <arch/arch.h>
#include <arch/config.h>

.text

.global OSTimer0_IRQ_Init
.global OSTimer0_IRQ_Enable
.global OSTimer0_IRQ_Disable
.global OSTimer0_IRQ


OSTimer0_IRQ_Init:
	stmfd	sp!, {r1-r2, lr}

//----------------- OSTimer0 interrupt setting
	ldr	r1, =INT_BASE

	// Interrupt Controller Control Register set Enable
	ldr	r2, [r1, #INT_ICCR]
	mov	r2, #0x0
	str	r2, [r1, #INT_ICCR]

	// Interrupt Controller Level Register set IRQ
	ldr	r2, [r1, #INT_ICLR]
	bic	r2, r2, #0x04000000		// 0 : IRQ,  1 : FIQ
	str	r2, [r1, #INT_ICLR]

	// Interrupt Controller Mask Register set Masked
	ldr	r2, [r1, #INT_ICMR]
	orr	r2, r2, #0x04000000		// 0 : masked,  1 : allowed
	str	r2, [r1, #INT_ICMR]

//----------------- OSTimer0 setting
	ldr	r1, =TMR_BASE

	// OSTimer0 Match Register reset
	ldr	r2, =_OSTIMER0_INTERVAL
	str	r2, [r1, #TMR_OSMR0]

	// OSTimer0 Interrupt Enable Register set
	mov	r2, #0x1
	str	r2, [r1, #TMR_OIER]

	// OSTimer Watchdog Register set Disable
	mov	r2, #0x0
	str	r2, [r1, #TMR_OWER]

	// OSTimer Counter Register clear
	mov	r2, #0x0
	str	r2, [r1, #TMR_OSCR]

	// OSTimer0 Source Register clear 
	mov	r2, #0x0
	str	r2, [r1, #TMR_OSSR]

	ldmfd	sp!, {r1-r2, pc}


OSTimer0_IRQ_Enable:
	stmfd	sp!, {r1-r2, lr}

	// Interrupt Controller Mask Register set
	ldr	r1, =INT_BASE
	ldr	r2, [r1, #INT_ICMR]
	orr	r2, r2, #0x04000000
	str	r2, [r1, #INT_ICMR]

	ldmfd	sp!, {r1-r2, pc}


OSTimer0_IRQ_Disable:
	stmfd	sp!, {r1-r2, lr}

	// Interrupt Controller Mask Register clear
	ldr	r1, =INT_BASE
	ldr	r2, [r1, #INT_ICMR]
	bic	r2, r2, #0x04000000
	str	r2, [r1, #INT_ICMR]

	ldmfd	sp!, {r1-r2, pc}


OSTimer0_IRQ:
	ldr	r12, =TMR_BASE

;	// OSTimer0 Source Register set
	ldr	r11, [r12, #TMR_OSSR]
	orr	r11, r11, #0x1
	str	r11, [r12, #TMR_OSSR]

;	// OSTimer0 Match Register reset
	ldr	r11, [r12, #TMR_OSCR]
	ldr	r10, =_OSTIMER0_INTERVAL
	add	r11, r11, r10
	str	r11, [r12, #TMR_OSMR0]

;	// I-bit set
	mrs	r12, spsr
	orr	r12, r12, #0x80
	msr	spsr_cxsf, r12

;	// stack pointer backup
	mov	r12, sp
	sub	sp, sp, #56		;// 14*4

;	// change from IRQ mode to SYS mode
	ldr	r0, =_Exit_irq
	movs	pc, r0

