
#include <arch/arch.h>
#include <drivers/key/key.h>


void enqueue_key(char);
char dequeue_key();
int next_queue(int);
int prev_queue(int);


static char key_buffer[MAX_QUEUE];
static int key_in_pos = 0;
static int key_out_pos = 0;
static int key_count = 0;

void key_init(void)
{
//----------------- GPIO[0] interrupt setting
	// Interrupt Controller Level Register set IRQ
	INT_REG(INT_ICLR) &= ~(BIT8);

	// Interrupt Controller Mask Register set Masked
	INT_REG(INT_ICMR) |= (BIT8);

//----------------- GPIO[0] setting
	// GPIO Pin Direction Register clear input
	GPIO_REG(GPDR0) &= ~(GPIO_0);

	// GPIO Alternate Function Register clear for normal GPIO function
	GPIO_REG(GAFR0_L) &= ~(0x3);

	// GPIO Edge Detect Status Register set
	GPIO_REG(GEDR0) |= (GPIO_0);

	// GPIO Rising Edge Detect Status Register clear
	GPIO_REG(GRER0) &= ~(GPIO_0);

	// GPIO Falling Edge Detect Status Register set
	GPIO_REG(GFER0) |= (GPIO_0);
}

void key_handler(void) 
{
	char key_value;

	// GPIO Edge Detect Status Register set
	GPIO_REG(GEDR0) |= (GPIO_0);

	key_value = (char)(KEY_REG & 0xF);
	enqueue_key(key_value);
}

char key_read(void)
{
	char value = -1;

	while (key_count == 0);
	value = dequeue_key();

	return value;
}

void enqueue_key(char value)
{
	int prev_key_out_pos;

	prev_key_out_pos = prev_queue(key_out_pos);

	if (key_count != MAX_QUEUE - 1) {
		key_buffer[key_in_pos] = value;
		key_in_pos = next_queue(key_in_pos);
		key_count++;
	}
}

char dequeue_key()
{
	char value;

	if (key_count != 0) {
		value = key_buffer[key_out_pos];
		key_out_pos = next_queue(key_out_pos);
		key_count--;
	}
	return value;
}

int next_queue(int pos)
{
	if (pos == MAX_QUEUE - 1)
		pos = 0;
	else
		pos++;

	return pos;
}

int prev_queue(int pos)
{
	if (pos == 0)
		pos = MAX_QUEUE - 1;
	else
		pos++;

	return pos;
}

