//==========================================================================
//
//      pxa255_lcd.c
//
// Description: Init Code for PXA255 LCD Controller
// target board : pxa255-pro V1.0  (Monitoring Program Ver1.X)               
// huins in 20004.6.            
// pjk is park joon ho , and all right is yours !! 
//==========================================================================

#include <drivers/lcd/types.h>
#include <drivers/lcd/lcd.h>
#include <drivers/lcd/font8x16.h>
#include <drivers/lcd/util.h>
//#include <drivers/lcd/stdio.h>
#include <drivers/lcd/lcdapp.h>
#include <arch/arch.h>

// globals to keep track of foreground, background colors and x,y position
int *lcd_color_depth_temp;
int lcd_color_depth;                // 4, 8 or 16
int lcd_fg_color;                       // 0 to 15, used as lookup into VGA color table
int lcd_bg_color;                       // 0 to 15, used as lookup into VGA color table
int lcd_col;                            // current column, 0 to COLS_PER_SCREEN - 1
int lcd_row;                            // current row, 0 to (ROWS_PER_SCREEN * 2) - 1
int lcd_tst_mode ;
ulong lcd_fb_offset;                    // current offset into frame buffer for lcd_putchar
extern int LcdMainFlag;
extern int Vout_Flag,FramBuffCnt;
int crv, cbu, cgu, cgv;

uchar framebuf0[640*2*480], framebuf1[640*2*480];
uchar *framebuf, *fbbackup;
uchar frameflag;

#if TFT_LCD_640X480
#define PIXELS_PER_ROW                  640
#define PIXELS_PER_COL                  480

#define ROWS_PER_SCREEN                 30
#define COLS_PER_SCREEN                 80
#endif

//#define LCD_BG_DEF                      100
//#define LCD_FG_DEF                      254
#define LCD_BG_DEF                      0xFFFFFFFF
#define LCD_FG_DEF                      0x00000000

#define LCD_FB_SIZE(_depth_)                            (((PIXELS_PER_COL * PIXELS_PER_ROW) * _depth_) / 8)
#define LCD_ROW_SIZE(_depth_)                           ((PIXELS_PER_ROW * _depth_) / 8)
#define LCD_GET_ADD(_row_, _col_, _depth_)      (((((_row_ * PIXELS_PER_ROW) * FONT_HEIGHT) + (_col_ * FONT_WIDTH)) * _depth_) / 8)
#define limit(x) (unsigned char)(((x) < 0) ? 0 : ((x) > 255) ? 255 : (x))

#define NIBBLES_PER_WORD	(8)

const ushort vga_lookup[] = {
LU_BLACK,		
LU_BLUE,			
LU_GREEN,		
LU_CYAN,			
LU_RED,			
LU_VIOLET,		
LU_YELLOW,		
LU_GREY,			
LU_WHITE,		
LU_BRT_BLUE,		
LU_BRT_GREEN,	
LU_BRT_CYAN,		
LU_BRT_RED,		
LU_BRT_VIOLET,
LU_BRT_YELLOW,
LU_BRT_WHITE	
};

const uchar lcd_lut[256][3] = {
//    RED,  GREEN, BLUE		// Entry
	{ 0xF0,  0x00, 0x00, },	// 00  
	{ 0x00,  0x00, 0xA0, },	// 01 
	{ 0x00,  0xA0, 0x00, },	// 02 
	{ 0x00,  0xA0, 0xA0, },	// 03 
	{ 0xA0,  0x00, 0x00, },	// 04 
	{ 0xA0,  0x00, 0xA0, },	// 05 
	{ 0xA0,  0xA0, 0x00, },	// 06 
	{ 0xA0,  0xA0, 0xA0, },	// 07 
	{ 0x50,  0x50, 0x50, },	// 08 
	{ 0x50,  0x50, 0xF0, },	// 09 
	{ 0x50,  0xF0, 0x50, },	// 0A 
	{ 0x50,  0xF0, 0xF0, },	// 0B 
	{ 0xF0,  0x50, 0x50, },	// 0C 
	{ 0xF0,  0x50, 0xF0, },	// 0D 
	{ 0xF0,  0xF0, 0x50, },	// 0E 
	{ 0xF0,  0xF0, 0xF0, },	// 0F 
	{ 0x00,  0x00, 0x00, },	// 10 
	{ 0x10,  0x10, 0x10, },	// 11 
	{ 0x20,  0x20, 0x20, },	// 12 
	{ 0x20,  0x20, 0x20, },	// 13 
	{ 0x30,  0x30, 0x30, },	// 14 
	{ 0x40,  0x40, 0x40, },	// 15 
	{ 0x50,  0x50, 0x50, },	// 16 
	{ 0x60,  0x60, 0x60, },	// 17 
	{ 0x70,  0x70, 0x70, },	// 18 
	{ 0x80,  0x80, 0x80, },	// 19 
	{ 0x90,  0x90, 0x90, },	// 1A 
	{ 0xA0,  0xA0, 0xA0, },	// 1B 
	{ 0xB0,  0xB0, 0xB0, },	// 1C 
	{ 0xC0,  0xC0, 0xC0, },	// 1D 
	{ 0xE0,  0xE0, 0xE0, },	// 1E 
	{ 0xF0,  0xF0, 0xF0, },	// 1F 
	{ 0x00,  0x00, 0xF0, },	// 20 
	{ 0x40,  0x00, 0xF0, },	// 21 
	{ 0x70,  0x00, 0xF0, },	// 22 
	{ 0xB0,  0x00, 0xF0, },	// 23 
	{ 0xF0,  0x00, 0xF0, },	// 24 
	{ 0xF0,  0x00, 0xB0, },	// 25 
	{ 0xF0,  0x00, 0x70, },	// 26 
	{ 0xF0,  0x00, 0x40, },	// 27 
	{ 0xF0,  0x00, 0x00, },	// 28 
	{ 0xF0,  0x40, 0x00, },	// 29 
	{ 0xF0,  0x70, 0x00, },	// 2A 
	{ 0xF0,  0xB0, 0x00, },	// 2B 
	{ 0xF0,  0xF0, 0x00, },	// 2C 
	{ 0xB0,  0xF0, 0x00, },	// 2D 
	{ 0x70,  0xF0, 0x00, },	// 2E 
	{ 0x40,  0xF0, 0x00, },	// 2F 
	{ 0x00,  0xF0, 0x00, },	// 30 
	{ 0x00,  0xF0, 0x40, },	// 31 
	{ 0x00,  0xF0, 0x70, },	// 32 
	{ 0x00,  0xF0, 0xB0, },	// 33 
	{ 0x00,  0xF0, 0xF0, },	// 34 
	{ 0x00,  0xB0, 0xF0, },	// 35 
	{ 0x00,  0x70, 0xF0, },	// 36 
	{ 0x00,  0x40, 0xF0, },	// 37 
	{ 0x70,  0x70, 0xF0, },	// 38 
	{ 0x90,  0x70, 0xF0, },	// 39 
	{ 0xB0,  0x70, 0xF0, },	// 3A 
	{ 0xD0,  0x70, 0xF0, },	// 3B 
	{ 0xF0,  0x70, 0xF0, },	// 3C 
	{ 0xF0,  0x70, 0xD0, },	// 3D 
	{ 0xF0,  0x70, 0xB0, },	// 3E 
	{ 0xF0,  0x70, 0x90, },	// 3F 
	{ 0xF0,  0x70, 0x70, },	// 40 
	{ 0xF0,  0x90, 0x70, },	// 41 
	{ 0xF0,  0xB0, 0x70, },	// 42 
	{ 0xF0,  0xD0, 0x70, },	// 43 
	{ 0xF0,  0xF0, 0x70, },	// 44 
	{ 0xD0,  0xF0, 0x70, },	// 45 
	{ 0xB0,  0xF0, 0x70, },	// 46 
	{ 0x90,  0xF0, 0x70, },	// 47 
	{ 0x70,  0xF0, 0x70, },	// 48 
	{ 0x70,  0xF0, 0x90, },	// 49 
	{ 0x70,  0xF0, 0xB0, },	// 4A 
	{ 0x70,  0xF0, 0xD0, },	// 4B 
	{ 0x70,  0xF0, 0xF0, },	// 4C 
	{ 0x70,  0xD0, 0xF0, },	// 4D 
	{ 0x70,  0xB0, 0xF0, },	// 4E 
	{ 0x70,  0x90, 0xF0, },	// 4F 
	{ 0xB0,  0xB0, 0xF0, },	// 50 
	{ 0xC0,  0xB0, 0xF0, },	// 51 
	{ 0xD0,  0xB0, 0xF0, },	// 52 
	{ 0xE0,  0xB0, 0xF0, },	// 53 
	{ 0xF0,  0xB0, 0xF0, },	// 54 
	{ 0xF0,  0xB0, 0xE0, },	// 55 
	{ 0xF0,  0xB0, 0xD0, },	// 56 
	{ 0xF0,  0xB0, 0xC0, },	// 57 
	{ 0xF0,  0xB0, 0xB0, },	// 58 
	{ 0xF0,  0xC0, 0xB0, },	// 59 
	{ 0xF0,  0xD0, 0xB0, },	// 5A 
	{ 0xF0,  0xE0, 0xB0, },	// 5B 
	{ 0xF0,  0xF0, 0xB0, },	// 5C 
	{ 0xE0,  0xF0, 0xB0, },	// 5D 
	{ 0xD0,  0xF0, 0xB0, },	// 5E 
	{ 0xC0,  0xF0, 0xB0, },	// 5F 
	{ 0xB0,  0xF0, 0xB0, },	// 60 
	{ 0xB0,  0xF0, 0xC0, },	// 61 
	{ 0xB0,  0xF0, 0xD0, },	// 62 
	{ 0xB0,  0xF0, 0xE0, },	// 63 
	{ 0xB0,  0xF0, 0xF0, },	// 64 
	{ 0xB0,  0xE0, 0xF0, },	// 65 
	{ 0xB0,  0xD0, 0xF0, },	// 66 
	{ 0xB0,  0xC0, 0xF0, },	// 67 
	{ 0x00,  0x00, 0x70, },	// 68 
	{ 0x10,  0x00, 0x70, },	// 69 
	{ 0x30,  0x00, 0x70, },	// 6A 
	{ 0x50,  0x00, 0x70, },	// 6B 
	{ 0x70,  0x00, 0x70, },	// 6C 
	{ 0x70,  0x00, 0x50, },	// 6D 
	{ 0x70,  0x00, 0x30, },	// 6E 
	{ 0x70,  0x00, 0x10, },	// 6F 
	{ 0x70,  0x00, 0x00, },	// 70 
	{ 0x70,  0x10, 0x00, },	// 71 
	{ 0x70,  0x30, 0x00, },	// 72 
	{ 0x70,  0x50, 0x00, },	// 73 
	{ 0x70,  0x70, 0x00, },	// 74 
	{ 0x50,  0x70, 0x00, },	// 75 
	{ 0x30,  0x70, 0x00, },	// 76 
	{ 0x10,  0x70, 0x00, },	// 77 
	{ 0x00,  0x70, 0x00, },	// 78 
	{ 0x00,  0x70, 0x10, },	// 79 
	{ 0x00,  0x70, 0x30, },	// 7A 
	{ 0x00,  0x70, 0x50, },	// 7B 
	{ 0x00,  0x70, 0x70, },	// 7C 
	{ 0x00,  0x50, 0x70, },	// 7D 
	{ 0x00,  0x30, 0x70, },	// 7E 
	{ 0x00,  0x10, 0x70, },	// 7F 
	{ 0x30,  0x30, 0x70, },	// 80 
	{ 0x40,  0x30, 0x70, },	// 81 
	{ 0x50,  0x30, 0x70, },	// 82 
	{ 0x60,  0x30, 0x70, },	// 83 
	{ 0x70,  0x30, 0x70, },	// 84 
	{ 0x70,  0x30, 0x60, },	// 85 
	{ 0x70,  0x30, 0x50, },	// 86 
	{ 0x70,  0x30, 0x40, },	// 87 
	{ 0x70,  0x30, 0x30, },	// 88 
	{ 0x70,  0x40, 0x30, },	// 89 
	{ 0x70,  0x50, 0x30, },	// 8A 
	{ 0x70,  0x60, 0x30, },	// 8B 
	{ 0x70,  0x70, 0x30, },	// 8C 
	{ 0x60,  0x70, 0x30, },	// 8D 
	{ 0x50,  0x70, 0x30, },	// 8E 
	{ 0x40,  0x70, 0x30, },	// 8F 
	{ 0x30,  0x70, 0x30, },	// 90 
	{ 0x30,  0x70, 0x40, },	// 91 
	{ 0x30,  0x70, 0x50, },	// 92 
	{ 0x30,  0x70, 0x60, },	// 93 
	{ 0x30,  0x70, 0x70, },	// 94 
	{ 0x30,  0x60, 0x70, },	// 95 
	{ 0x30,  0x50, 0x70, },	// 96 
	{ 0x30,  0x40, 0x70, },	// 97 
	{ 0x50,  0x50, 0x70, },	// 98 
	{ 0x50,  0x50, 0x70, },	// 99 
	{ 0x60,  0x50, 0x70, },	// 9A 
	{ 0x60,  0x50, 0x70, },	// 9B 
	{ 0x70,  0x50, 0x70, },	// 9C 
	{ 0x70,  0x50, 0x60, },	// 9D 
	{ 0x70,  0x50, 0x60, },	// 9E 
	{ 0x70,  0x50, 0x50, },	// 9F 
	{ 0x70,  0x50, 0x50, },	// A0 
	{ 0x70,  0x50, 0x50, },	// A1 
	{ 0x70,  0x60, 0x50, },	// A2 
	{ 0x70,  0x60, 0x50, },	// A3 
	{ 0x70,  0x70, 0x50, },	// A4 
	{ 0x60,  0x70, 0x50, },	// A5 
	{ 0x60,  0x70, 0x50, },	// A6 
	{ 0x50,  0x70, 0x50, },	// A7 
	{ 0x50,  0x70, 0x50, },	// A8 
	{ 0x50,  0x70, 0x50, },	// A9 
	{ 0x50,  0x70, 0x60, },	// AA 
	{ 0x50,  0x70, 0x60, },	// AB 
	{ 0x50,  0x70, 0x70, },	// AC 
	{ 0x50,  0x60, 0x70, },	// AD 
	{ 0x50,  0x60, 0x70, },	// AE 
	{ 0x50,  0x50, 0x70, },	// AF 
	{ 0x00,  0x00, 0x40, },	// B0 
	{ 0x10,  0x00, 0x40, },	// B1 
	{ 0x20,  0x00, 0x40, },	// B2 
	{ 0x30,  0x00, 0x40, },	// B3 
	{ 0x40,  0x00, 0x40, },	// B4 
	{ 0x40,  0x00, 0x30, },	// B5 
	{ 0x40,  0x00, 0x20, },	// B6 
	{ 0x40,  0x00, 0x10, },	// B7 
	{ 0x40,  0x00, 0x00, },	// B8 
	{ 0x40,  0x10, 0x00, },	// B9 
	{ 0x40,  0x20, 0x00, },	// BA 
	{ 0x40,  0x30, 0x00, },	// BB 
	{ 0x40,  0x40, 0x00, },	// BC 
	{ 0x30,  0x40, 0x00, },	// BD 
	{ 0x20,  0x40, 0x00, },	// BE 
	{ 0x10,  0x40, 0x00, },	// BF 
	{ 0x00,  0x40, 0x00, },	// C0 
	{ 0x00,  0x40, 0x10, },	// C1 
	{ 0x00,  0x40, 0x20, },	// C2 
	{ 0x00,  0x40, 0x30, },	// C3 
	{ 0x00,  0x40, 0x40, },	// C4 
	{ 0x00,  0x30, 0x40, },	// C5 
	{ 0x00,  0x20, 0x40, },	// C6 
	{ 0x00,  0x10, 0x40, },	// C7 
	{ 0x20,  0x20, 0x40, },	// C8 
	{ 0x20,  0x20, 0x40, },	// C9 
	{ 0x30,  0x20, 0x40, },	// CA 
	{ 0x30,  0x20, 0x40, },	// CB 
	{ 0x40,  0x20, 0x40, },	// CC 
	{ 0x40,  0x20, 0x30, },	// CD 
	{ 0x40,  0x20, 0x30, },	// CE 
	{ 0x40,  0x20, 0x20, },	// CF 
	{ 0x40,  0x20, 0x20, },	// D0 
	{ 0x40,  0x20, 0x20, },	// D1 
	{ 0x40,  0x30, 0x20, },	// D2 
	{ 0x40,  0x30, 0x20, },	// D3 
	{ 0x40,  0x40, 0x20, },	// D4 
	{ 0x30,  0x40, 0x20, },	// D5 
	{ 0x30,  0x40, 0x20, },	// D6 
	{ 0x20,  0x40, 0x20, },	// D7 
	{ 0x20,  0x40, 0x20, },	// D8 
	{ 0x20,  0x40, 0x20, },	// D9 
	{ 0x20,  0x40, 0x30, },	// DA 
	{ 0x20,  0x40, 0x30, },	// DB 
	{ 0x20,  0x40, 0x40, },	// DC 
	{ 0x20,  0x30, 0x40, },	// DD 
	{ 0x20,  0x30, 0x40, },	// DE 
	{ 0x20,  0x20, 0x40, },	// DF 
	{ 0x20,  0x20, 0x40, },	// E0 
	{ 0x30,  0x20, 0x40, },	// E1 
	{ 0x30,  0x20, 0x40, },	// E2 
	{ 0x30,  0x20, 0x40, },	// E3 
	{ 0x40,  0x20, 0x40, },	// E4 
	{ 0x40,  0x20, 0x30, },	// E5 
	{ 0x40,  0x20, 0x30, },	// E6 
	{ 0x40,  0x20, 0x30, },	// E7 
	{ 0x40,  0x20, 0x20, },	// E8 
	{ 0x40,  0x30, 0x20, },	// E9 
	{ 0x40,  0x30, 0x20, },	// EA 
	{ 0x40,  0x30, 0x20, },	// EB 
	{ 0x40,  0x40, 0x20, },	// EC 
	{ 0x30,  0x40, 0x20, },	// ED 
	{ 0x30,  0x40, 0x20, },	// EE 
	{ 0x30,  0x40, 0x20, },	// EF 
	{ 0x20,  0x40, 0x20, },	// F0 
	{ 0x20,  0x40, 0x30, },	// F1 
	{ 0x20,  0x40, 0x30, },	// F2 
	{ 0x20,  0x40, 0x30, },	// F3 
	{ 0x20,  0x40, 0x40, },	// F4 
	{ 0x20,  0x30, 0x40, },	// F5 
	{ 0x20,  0x30, 0x40, },	// F6 
	{ 0x20,  0x30, 0x40, },	// F7 
	{ 0x00,  0x00, 0x00, },	// F8 
	{ 0x00,  0x00, 0x00, },	// F9 
	{ 0x00,  0x00, 0x00, },	// FA 
	{ 0x00,  0x00, 0x00, },	// FB 
	{ 0x00,  0x00, 0x00, },	// FC 
	{ 0x00,  0x00, 0x00, },	// FD 
	{ 0x00,  0x00, 0x00, },	// FE 
	{ 0x00,  0x00, 0x00, },	// FF 
};

// pjk 030925

const uchar lcd_lut1[16][3] = {
//    	  RED,  GREEN, BLUE		// Entry
	{ 0x00,  0x00, 0x00, },	// 00  
	{ 0xFF,  0x00, 0x00, },	// 01 
	{ 0x00,  0xFF, 0x00, },	// 02 
	{ 0x00,  0x00, 0xFF, },	// 03 
	{ 0xFF,  0xFF, 0x00, },	// 04 
	{ 0xFF,  0x00, 0xFF, },	// 05 
	{ 0x00,  0xFF, 0xFF, },	// 06 
	{ 0x88,  0x00, 0x00, },	// 07 
	{ 0x00,  0x88, 0x00, },	// 08 
	{ 0x00,  0x00, 0x88, },	// 09 
	{ 0x88,  0x88, 0x00, },	// 0A 
	{ 0x00,  0x88, 0x88, },	// 0B 
	{ 0x88,  0x00, 0x88, },	// 0C 
	{ 0x88,  0x88, 0x88, },	// 0D 
	{ 0xAA,  0xAA, 0xAA, },	// 0E 
	{ 0xFF,  0xFF, 0xFF, },	// 0F 
};






//--------------------------------------------------------------------------
// lcd_init
//
// This function sets up the PXA250 LCD controller
//
int lcd_init(int lcd_depth_value)
{
//added by Hwang M.K.
LCD_START_BUF = LCD_BUF_ADD+3838;
  //  lcd_depth_value=&lcd_color_depth;
  //  lcd_color_depth_temp=&lcd_color_depth;
  //  *lcd_color_depth_temp=lcd_depth_value;
  //lcd_color_depth is 16
  if(lcd_depth_value==LCD_COLOR_DEPTH16)
    {
      lcd_color_depth=LCD_COLOR_DEPTH16;    
    }
  else
    {
      lcd_color_depth=LCD_COLOR_DEPTH8;
    }

        ulong index;
        int i;
        ushort temp16;

	lcd_tst_mode = 0;
        //printf("PXA255 LCD Initialization.\n");

        // Setup Timing Registers
        // Note, LCD pixel clock is derived from MEMCLK which
        // defaults to 99.5Mhz.  Any other value requires changing
        // LCCR3_PCD.
        LCD_REG(LCD_LCCR3) =  LCCR3_PCP			        // Pixel Clock Polarity, 1 = LCD Data is Sampled on Falling Edge
                                           | LCCR3_HSP		// Horizontal Sync Polarity, 1 = Active Low
                                           | LCCR3_VSP		// Vertical Sync Polarity, 1 = Active Low
			  		   | LCCR3_PCD(2);	// Pixel Clock Divisor, PCLK = LCLK/(2*(PCD+1)) = 99.5Mhz/4 = 25Mhz
	     	
        // set the bpp based on lcd_color_depth
        switch (lcd_color_depth)
        {
                case 4:
                        LCD_REG(LCD_LCCR3) |= LCCR3_BPP_4;      // 4-bits/pixel
                        break;
                case 8:
                        LCD_REG(LCD_LCCR3) |= LCCR3_BPP_8;      // 8-bits/pixel
                        break;
                default:
                        LCD_REG(LCD_LCCR3) |= LCCR3_BPP_16;     // 16-bits/pixel
                        break;
        } // switch color depth
        
#if TFT_LCD_640X480  //prime view(Model :PD064VT2)
        LCD_REG(LCD_LCCR2) = LCCR2_BFW(0x21)                                                       // Beginning-of-frame line clock wait count
                                           | LCCR2_EFW(0x5)                                        // End-of-frame line clock wait count
                                           | LCCR2_VSW(0x21)                                       // Vertical sync pulse width = VSW + 1
                                           | LCCR2_LPP(PIXELS_PER_COL - 1);                        // Lines per panel = LPP + 1

        LCD_REG(LCD_LCCR1) = LCCR1_BLW(0x51)                                                       // Beginning-of-Line pixel clock Wait count = BLW + 1
                                   | LCCR1_ELW(0x0f)                                               // End-of-line pixel clock wait count = ELW + 1
                                   | LCCR1_HSW(0x3f)                                               // Horizontal sync pulse width = HSW + 1
                                   | LCCR1_PPL(PIXELS_PER_ROW - 1);                                // Pixels per line = PPL + 1
#endif

        index = 0;
	// Fill the pallette buffer with the lookup table data

	/*
	if(lcd_color_depth == 4){
		       for (i = 0; i < 16; i ++){
                		LCD_PAL(index) = (((lcd_lut1[i][0] & 0xf8) << 8)         // Red
                                | ((lcd_lut1[i][1] & 0xfc) << 3)     			 // Green
                                | ((lcd_lut1[i][2]) >> 3));                 		 // Blue

                		index += 2;
        		}
        printf(".....................................\n");
        }
        
        else {	          
        		for (i = 0; i < 256; i ++){
                		LCD_PAL(index) = (((lcd_lut[i][0] & 0xf8) << 8)         // Red
                                | ((lcd_lut[i][1] & 0xfc) << 3)                         // Green
                                | ((lcd_lut[i][2]) >> 3));                              // Blue

                		index += 2;
        		}
			}
*/
	// Fill the palette buffer with the lookup table tata
	if(lcd_color_depth==LCD_COLOR_DEPTH8||lcd_color_depth==LCD_COLOR_DEPTH16)
	  {
	    for (i = 0; i < 256; i ++){
	      LCD_PAL(index) = (((lcd_lut[i][0] & 0xf8) << 8)         // Red
                                | ((lcd_lut[i][1] & 0xfc) << 3)                         // Green
                                | ((lcd_lut[i][2]) >> 3));                              // Blue
	      
	      index += 2;
	    }	
	  }
        // Setup the final LCD control register, but
        // leave the controller off.  It will be controlled 
        // by lcd_on() and lcd_off() which also handles the
        // backlight
        LCD_REG(LCD_LCCR0) = LCCR0_OUM          // Output FIFO Underrun Mask, 1 = no interrupt
                                           | LCCR0_BM           // Branch Mask, 1 = no interrupt
                                           | LCCR0_QDM          // Quick Disable Interrupt Mask, 1 = no interrupt
                                           | LCCR0_PAS          // Passive/Active Display select 1 = TFT
                                           | LCCR0_EFM          // End of Frame Interrupt Mask, 1 = no interrupt
                                           | LCCR0_IUM          // Input FIFO Underrun Mask, 1 = no interrupt
                                           | LCCR0_SFM          // Start of Frame Mask, 1 = no interrupt
                                           | LCCR0_LDM;         // LCD Disable Done Mask, 1 = no interrupt
        // initialize the globals       
        lcd_fg_color = LCD_FG_DEF;
        lcd_bg_color = LCD_BG_DEF;
        lcd_row = 0;
        lcd_col = 0;
        lcd_fb_offset = 0x00;

        // fill the screen with the background color, remember for the console driver
        // we do double buffering, so fill twice as mush as the active screen size
        switch (lcd_color_depth){
                case 4:  temp16 = lcd_bg_color | lcd_bg_color << 4 | lcd_bg_color << 8 | lcd_bg_color << 12; break;
                case 8:  temp16 = lcd_bg_color | lcd_bg_color << 8; break;
                default: temp16 = lcd_bg_color; break;
        }
        for (i = 0; i < (PIXELS_PER_COL * PIXELS_PER_ROW) * 2; i += 2){ 
                LCD_BUF(i) = temp16;
        }
        return 0;
}


//--------------------------------------------------------------------------
// lcd_on
//
// This function turns on the PXA250 LCD Controller and the Backlight Inverter
//
void lcd_on(void)
{
//added by Hwang M.K.
LCD_START_BUF = LCD_BUF_ADD+3838;


        // Make sure it's off first
        // lcd_off();

        // Turn on the LCD Controller
        // LCD_REG(LCD_LCCR0) |= (LCD_REG(LCD_LCCR0) | LCCR0_ENB); 

        // Enable the Backlight
        // GPIO_set(LCD_BKL);
        GPIO_REG(GPSR0) = (1 << LCD_BKL); //mwk

        // kick off the frame descriptors
        // Setup the Palette Descriptor
        LCD_PDR(0x0) = LCD_FDR_ADD;                               // point to the frame buffer descriptor
        LCD_PDR(0x4) = LCD_PAL_ADD;                               // beginning of Palette Buffer in Memory
        LCD_PDR(0x8) = 0;                                         // Frame ID
        LCD_PDR(0xc) = BIT26 | 512;                               // PAL = 1, Frame Length in bytes = 512

        // Setup the Frame Descriptor
        LCD_FDR(0x0) = LCD_FDR_ADD;                               // point it to itself
        LCD_FDR(0x4) = LCD_BUF_ADD;                               // beginning of Frame Buffer in Memory
       
        LCD_FDR(0x8) = 0;                                         // Frame ID
        LCD_FDR(0xc) = LCD_FB_SIZE(lcd_color_depth);              // Frame Length in bytes

        
        // For color depths of 4 or 8 we point the PXA250 to 
        // the palette buffer descriptor, which will then point
        // to the frame buffer descriptor, while for 16 bpp 
        // we point directly to the frame buffer descriptor,
        // thus skipping the palette buffer descriptor.
        if ((lcd_color_depth == LCD_COLOR_DEPTH4) || (lcd_color_depth == LCD_COLOR_DEPTH8)){
                LCD_FDR(0x0) = LCD_PDR_ADD;    
                // point back to the palette descriptor 
                LCD_REG(LCD_FDADR0) = LCD_PDR_ADD;
        }
        else {
              //LCD_FDR(0x0) = LCD_FDR_ADD;                       // point it to itself as it's the only descriptor
                LCD_REG(LCD_FDADR0) = LCD_FDR_ADD;
        }
	   LCD_REG(LCD_LCCR0) |= (LCD_REG(LCD_LCCR0) | LCCR0_ENB); 
        // wait for power-up sequence to complete
	int i=0;
	int j=0;
	for(;i<100000;i++);
		j++;
	
}

//--------------------------------------------------------------------------
// lcd_off
//
// This function turns off the LCD Controller and the LCD backlight
//
void lcd_off()
{
        // Turn off the LCD Controller using the diable bit
        LCD_REG(LCD_LCCR0) |= (LCD_REG(LCD_LCCR0) | LCCR0_DIS);
        
        // Wait for the Enable Bit to go clear
        
        //while(LCD_REG(LCD_LCCR0) & LCCR0_ENB) {}       
        // Turn off the backlight
	// GPIO_clr(LCD_BKL);
	GPIO_REG(GPCR0) = (1 << LCD_BKL);  // mwk
	
}


#if 0
int lcd_tst(int argc,char *argv[])
{
  volatile ushort wr16;//color;
  volatile uchar color8;
  volatile ushort color16;

  int start_point;//,start_x,start_y,temp_xy;
	int i,x=0;
	int col=0,row=0,counter=0;
	int no_wait = 0;
	int init_only = 0;


	if(argc!=5){
	  printf("Usage: lcdtest [col] [row] [color] [counter]\n");
	  return 0;}
	col=aToi(argv[1]);
	row=aToi(argv[2]);	
	color8=aToi(argv[3]);
	color16=aToi(argv[3]);
	counter=aToi(argv[4]);

        lcd_tst_mode = 1;
	lcd_off();

	//    	lcd_color_depth = 16;
	if(lcd_color_depth==LCD_COLOR_DEPTH16)
	  {
	    lcd_color_depth=LCD_COLOR_DEPTH16;
	  }
	printf("Pre This lcd_test depth value: %d\n\n\n\n\n\n",lcd_color_depth);
        lcd_init(lcd_color_depth);
	printf("For This lcd_test depth value: %d\n\n\n\n\n\n",lcd_color_depth);

        lcd_on();

        if (init_only) return 0;

	//        printf("Frame Buffer R/W...");
        // do an address=data read/write test on the frame buffer
        // PIXELS_PER_COL * PIXELS_PER_ROW is the highest pixel.  
        // Multiply by bits_per_pixel (sed_color_depth), then
        // divide by 8 to get the actual byte count.
	/*        for (i = 0; i < LCD_FB_SIZE(lcd_color_depth); i += 2){  
                LCD_BUF(i) = i & 0xffff;
                rd16 = LCD_BUF(i);
                if(rd16 != (i & 0xffff)){
                //        printf("Fail at 0x%08x, WR 0x%08lx, RD 0x%04x!\n",LCD_BUF_ADD + i, i, rd16);
                        return -1;
                }
        } 
	*/

	//        printf("OK!, Press key to continue.\n");
	//	c = (char)SerialInput();

	//printf("Frame Buffer Start: 0x%08lx, End 0x%08lx\n",LCD_BUF_ADD, LCD_BUF_ADD + LCD_FB_SIZE(lcd_color_depth));
        if (no_wait)
        {
                printf("Begin Full Screen Color Test.\n");
                while(1){
                        // fill the frame buffer with incrementing color values
                        for (x = 0; x < 16; x++){
                                switch (lcd_color_depth){
                                        case 4:  wr16 = x | x << 4 | x << 8 | x << 12; break;
                                        case 8:  wr16 = x | x << 8; break;
                                        default: wr16 = vga_lookup[x]; break;   // 16-bits bypasses the lookup table
                                }
                                for (i = 0; i < LCD_FB_SIZE(lcd_color_depth); i += 2){  
				  LCD_BUF(i) = wr16;
				}
                        } // for x
                } // while
        } // no_wait
        else
	  {
	    printf("Begin Full Screen Color Test, Press any key to go to next color, \'x\' to end.\n");
		// fill the frame buffer with incrementing color values
		    /*
		    switch (lcd_color_depth)
		      {
		      case 4:  wr16 = x | x << 4 | x << 8 | x << 12; break;
		      case 8:  wr16 = x | x << 8; break;
		      default: wr16 = vga_lookup[x]; break;   // 16-bits bypasses the lookup table
		      }
		    */
		    wr16=200*x;

		    //for (i = 150000; i < LCD_FB_SIZE(lcd_color_depth); i += 2)
		    //  { LCD_BUF(i) = wr16;}

		    // start_point is 3838 = 0xEFE
		    // secont point is 5118
		    // one line is 1280
		    // color info: 5/6/5 system
		    // last_point is 614398 (fill col line--;)

		    //color depth is 8
		    if(lcd_color_depth==8)
		      {
			start_point=3838+col+640*row;
			printf("8 Start_point  X: %d  Y: %d  Color: %d  Counter: %d\n",col,row,color8,counter);
			for (i=0 ; i < counter ; i++ , start_point+=1)
			  { 
			    LCD_BUF_CHAR(start_point) = color8;
			    printf("8This Point : %d  X: %d  Y: %d  Color: %d\n",start_point,col,row,color8);
			    col=(col+1)%641;
			    if(col==0)
			      row++;
			  }
		      }
		    //color depth is 16
		    if(lcd_color_depth==16)
		      {
			start_point=3836+col*2+1280*row;
			printf("Start_point  X: %d  Y: %d  Color: %d  Counter: %d\n",col,row,color16,counter);
			for (i=0 ; i < counter ; i++ ,start_point+=2)
			  { 
			    LCD_BUF(start_point) = color16;
			    printf("16This Point: %d  X: %d  Y: %d  Color: %d\n",start_point,col,row,color16);
			    col=(col+1)%641;
			    if(col==0)
			      row++;
			  }
		      }


		    printf("Input next\n");
		    //		    c = (char)SerialInput();
		    
		    // pjk 2003.03.18 
		    //#if  !PJK_TEST
		    //		    if (c == 'x')  goto lcd_tst_next;
		    //		    if (c == 'X') {
		    //		      printf("lcd_data transfer start........."); 
		    //		      for (i = 0; i < LCD_FB_SIZE(lcd_color_depth); i +=2){
			
			
		    //			LCD_BUF(i) = (char)SerialInput();
			
		    //		      }                     
		    //		    }                        
		    //#else
		    //		    if (c == 'x')  goto lcd_tst_next;
		    //#endif
		    
	  } // else no keycheck test
	
	// lcd_tst_next:
	
	//        lcd_off();
	//        lcd_init(lcd_color_depth);
	//        lcd_on();
	//        lcd_tst_mode = 0;
		return 0;
}
#endif



// lcd_scroll()
//
// Because we are most likely running out of FLASH and probably also with
// cache disabled, a brute force memcpy of the whole screen would be very
// slow, even with reduced color depths.  Instead, we define a frame buffer
// that is twice the size of our actual display.
//
// With the double buffering, we always write to the first buffer, even
// when the second buffer is active.  This allows us to scroll by adjusting
// the starting and ending addresses in the PXA250 by one row.  When we
// reach the end of our virtual buffer, we reset the starting and ending
// addresses to the first buffer.
//
ulong lcd_scroll(){  

        lcd_row++;

        // clear the new row(s)
        lcd_clr_row(lcd_row);
        if (lcd_row > (ROWS_PER_SCREEN - 1)){
                lcd_clr_row(lcd_row - ROWS_PER_SCREEN);
        }
        // when lcd_y_pos is greater than ROWS_PER_SCREEN we just adjust the
        // start and end addresses in the PXA250.  If it is equal to 2 *
        // ROWS_PER_SCREEN, we reset the start and end addresses to LCD_MEM_BASE.
        if (lcd_row > (ROWS_PER_SCREEN - 1))
        {
                if (lcd_row > ((ROWS_PER_SCREEN * 2) - 1))
                {
                        lcd_fb_offset = 0x00;
                        lcd_row = ROWS_PER_SCREEN - 1;
                }
                else
                {
                        // calculate the new offset address of the frame buffer in words
                        lcd_fb_offset += (LCD_GET_ADD(1, 0, lcd_color_depth));
                }

#ifdef LCD_DBG
        lcd_tst_mode = 1;
        printf("LCD New FB Offset = 0x%08lx.\n ", lcd_fb_offset);
        lcd_tst_mode = 0;
#endif
                LCD_FDR(0x4) = LCD_BUF_ADD + lcd_fb_offset;                             // new beginning of Frame Buffer in Memory

        } // if (lcd_row > (ROWS_PER_SCREEN - 1))
        return 0 ;
                

} // 

//--------------------------------------------------------------------------
// lcd_clr_row()
//
// This routine writes the background color to the font row specified
//
void lcd_clr_row(int char_row){

        ulong lcd_mem_add;
        int i;
        ushort wr16;

        // clear the desired row
        lcd_mem_add = LCD_GET_ADD(char_row, 0, lcd_color_depth);

#ifdef LCD_DBG
        lcd_tst_mode = 1;
        printf("LCD Clear Row %d, FB Add 0x%08lx.\n ", char_row, lcd_mem_add);
        lcd_tst_mode = 0;
#endif

        switch (lcd_color_depth)
        {
                case 4:
                        wr16 = ((lcd_bg_color << 12) | (lcd_bg_color << 8) | (lcd_bg_color << 4) | (lcd_bg_color << 0));
                        for (i = 0; i < ((PIXELS_PER_ROW * FONT_HEIGHT) / 2); i += 2){
                                LCD_BUF(lcd_mem_add) = wr16;
                                lcd_mem_add += 2;
                        } // for font_row
                        break;
                case 8:
                        wr16 = ((lcd_bg_color << 8) | (lcd_bg_color << 0));
                        for (i = 0; i < (PIXELS_PER_ROW * FONT_HEIGHT); i += 2){
                                LCD_BUF(lcd_mem_add) = wr16;
                                lcd_mem_add += 2;
                        } // for font_row
                        break;
                case 16:
                        wr16 = (lcd_bg_color << 0);
                        for (i = 0; i < ((PIXELS_PER_ROW * FONT_HEIGHT) * 2); i += 2){
                                LCD_BUF(lcd_mem_add) = wr16;
                                lcd_mem_add += 2;
                        } // for font_row
                        break;
        } // switch lcd_color_depth
} // lcd_clr_row


//--------------------------------------------------------------------------
// lcd_putchar()
//
// This routine parses the character and calls lcd_writechar if it is a
// printable character
//
void lcd_putchar(char c){


        // First parse the character to see if it is printable or an acceptable control
    switch (c) {
            case '\r':
                lcd_col = 0;
                return;
            case '\n':
                lcd_scroll();
                return;
            case '\b':
                lcd_col--;
                if (lcd_col < 0)
                {
                    lcd_row--;
                    if (lcd_row < 0) lcd_row = 0;
                    lcd_col = COLS_PER_SCREEN - 1;
                    }
                        c = 0;          // erase the character
                        lcd_writechar(c);
                break;
            default:
                if (((uchar)c < FIRST_CHAR) || ((uchar)c > LAST_CHAR)) return; // drop anything we can't print
                        c -= FIRST_CHAR;        // get aligned to the first printable character
                        lcd_writechar(c);
                        // advance to next column
                        lcd_col++;
                    if (lcd_col == COLS_PER_SCREEN)
                    {
                                lcd_col = 0;
				lcd_scroll();
		    }
		    break;
    } 

} // lcd_putchar()

void lcd_fillbg(int color) {
  int i;
  ushort temp16;

        switch (lcd_color_depth){
                case 4:  temp16 = color | color << 4 | color << 8 | color << 12; break;
                case 8:  temp16 = color | color << 8; break;
                default: temp16 = color; break;
        }
        for (i = 0; i < (PIXELS_PER_COL * PIXELS_PER_ROW) * 2; i += 2){ 
                LCD_BUF(i) = temp16;
        }
}

//--------------------------------------------------------------------------
// lcd_writechar()
//
// This routine writes the character to the screen at the current cursor
// location.
//
void lcd_writechar(uchar c) {

        ulong lcd_mem_add;
        int font_row, font_col;
        uchar font_data8;
        ushort  wr16;

        // Convert the current row,col and color depth values 
        // into an address
        lcd_mem_add = LCD_GET_ADD(lcd_row, lcd_col, lcd_color_depth);



#ifdef LCD_DBG
        lcd_tst_mode = 1;
        printf("SED writechar at row %d, col %d, FB Add 0x%08lx.\n ", lcd_row, lcd_col, lcd_mem_add);
        lcd_tst_mode = 0;
#endif

        if (FONT_WIDTH == 8)
        {
                switch (lcd_color_depth){
                        case 4:
                                // Now render the font by painting one font row at a time
                                for (font_row = 0; font_row < FONT_HEIGHT; font_row++){
                                        // get the font row of data
                                        font_data8 = font8x16[(c * FONT_HEIGHT) + font_row];
                                        for (font_col = 0; font_col < 8; font_col += 4)
                                        {
                                                // get a words worth of pixels
                                                wr16 = (((font_data8 & 0x80) ? lcd_fg_color << 0 : lcd_bg_color << 0)
                                                          | ((font_data8 & 0x40) ? lcd_fg_color << 4 : lcd_bg_color << 4)
                                                          | ((font_data8 & 0x20) ? lcd_fg_color << 8 : lcd_bg_color << 8)
                                                          | ((font_data8 & 0x10) ? lcd_fg_color << 12 : lcd_bg_color << 12));
                                                font_data8 = font_data8 << 4;
//                                              wr16 = (((font_data8 & 0x80) ? lcd_fg_color << 12 : lcd_bg_color << 12)
//                                                        | ((font_data8 & 0x40) ? lcd_fg_color << 8 : lcd_bg_color << 8)
//                                                        | ((font_data8 & 0x20) ? lcd_fg_color << 4 : lcd_bg_color << 4)
//                                                        | ((font_data8 & 0x10) ? lcd_fg_color << 0 : lcd_bg_color << 0));
//                                              font_data8 = font_data8 << 4;
                                                LCD_BUF(lcd_mem_add) = wr16;
                                                // if we are in the 2nd frame buffer, write to the 1st frame buffer also
                                                if (lcd_row > (ROWS_PER_SCREEN - 1)){
                                                        LCD_BUF(lcd_mem_add - LCD_FB_SIZE(lcd_color_depth)) = wr16;
                                                }
                                                lcd_mem_add += 2;
                                        } // for font_col
                                        // go to the next pixel row
                                        lcd_mem_add += (LCD_ROW_SIZE(lcd_color_depth) - ((FONT_WIDTH * lcd_color_depth) / 8));
                                } // for font_row
                                break;

                        case 8:
                                // Now render the font by painting one font row at a time
                                for (font_row = 0; font_row < FONT_HEIGHT; font_row++){
                                        // get the font row of data
                                        font_data8 = font8x16[(c * FONT_HEIGHT) + font_row];
                                        for (font_col = 0; font_col < 8; font_col += 2)
                                        {
                                                // get a words worth of pixels
                                                wr16 = (((font_data8 & 0x80) ? lcd_fg_color << 0 : lcd_bg_color << 0)
                                                          | ((font_data8 & 0x40) ? lcd_fg_color << 8 : lcd_bg_color << 8));
                                                font_data8 = font_data8 << 2;
//                                              wr16 = (((font_data8 & 0x80) ? lcd_fg_color << 8 : lcd_bg_color << 8)
//                                                        | ((font_data8 & 0x40) ? lcd_fg_color << 0 : lcd_bg_color << 0));
//                                              font_data8 = font_data8 << 2;
                                                LCD_BUF(lcd_mem_add) = wr16;
                                                // if we are in the 2nd frame buffer, write to the 1st frame buffer also
                                                if (lcd_row > (ROWS_PER_SCREEN - 1)){
                                                        LCD_BUF(lcd_mem_add - LCD_FB_SIZE(lcd_color_depth)) = wr16;
                                                }
                                                lcd_mem_add += 2;
                                        } // for font_col
                                        // go to the next pixel row
                                        lcd_mem_add += (LCD_ROW_SIZE(lcd_color_depth) - ((FONT_WIDTH * lcd_color_depth) / 8));
                                } // for font_row
                                break;

                        default:        // 16bpp is the default
                                // Now render the font by painting one font row at a time
                                for (font_row = 0; font_row < FONT_HEIGHT; font_row++){
                                        // get the font row of data
                                        font_data8 = font8x16[(c * FONT_HEIGHT) + font_row];
                                        for (font_col = 0; font_col < 8; font_col++)
                                        {
                                                // get a words worth of pixels
                                                wr16 = ((font_data8 & 0x80) ? lcd_fg_color : lcd_bg_color);
                                                font_data8 = font_data8 << 1;
                                                LCD_BUF(lcd_mem_add) = wr16;
                                                // if we are in the 2nd frame buffer, write to the 1st frame buffer also
                                                if (lcd_row > (ROWS_PER_SCREEN - 1)){
                                                        LCD_BUF(lcd_mem_add - LCD_FB_SIZE(lcd_color_depth)) = wr16;
                                                }
                                                lcd_mem_add += 2;
                                        } // for font_col
                                        // go to the next pixel row
                                        lcd_mem_add += (LCD_ROW_SIZE(lcd_color_depth) - ((FONT_WIDTH * lcd_color_depth) / 8));
                                } // for font_row
                                break;
                } // switch lcd_color depth
        } // FONT_WIDTH == 8
        else
        {
                return;
        }
} // lcd_writechar()

void lcd_putstring(char *s) {
  char *ptr = s;

  while(*ptr != '\0') {
	lcd_putchar(*ptr);
	ptr++;
  }
}

void lcd_puthex(const unsigned long h)
{
	char c;
	int i;
	
	for (i = NIBBLES_PER_WORD - 1; i >= 0; i--) {
		c = (char)((h >> (i * 4)) & 0x0f);

		if (c > 9)
			c += ('a' - 10);
		else
			c += '0';

		lcd_putchar(c);
	}
}

void lcd_setpos(int col, int row) {
  lcd_col = col;
  lcd_row = row;
}

void lcd_setfgbg(int fg, int bg) {
  lcd_fg_color = fg;
  lcd_bg_color = bg;
}

// for 16bit mode
// option : option&0x10 == 0 -> put pixel to buffer frame directly.
//          option&0x10 == 2 -> put pixel using 2 framebuf array.
//          option&0x01 == 0 -> dont back up.
//          option&0x01 == 1 -> do back up.
void lcd_putpic(unsigned char *img, int posx, int posy, int width, int height, unsigned char option) {
  int row, col, temprow;
  ulong lcd_mem_add;

  width = width * 2;
  lcd_mem_add = posy*1280 + posx*2;  // 1280 = 640 * 2

  if((option&0x10) == 0x10) {
    if(frameflag == 0) {
	  frameflag = 1;
	  framebuf = framebuf0;
	  fbbackup = framebuf1;
    } else {
	  frameflag = 0;
	  framebuf = framebuf1;
	  fbbackup = framebuf0;
    }
  }

  for(row=0; row < height; row++) {
    temprow = row * width;
    for(col=0; col < width; col++) {
	  framebuf[lcd_mem_add] = img[temprow + col];
	  lcd_mem_add++;
	}
	lcd_mem_add += (1280 - width);
  }

  if((option&0x10) == 0x10)
    LcdChangePage(framebuf);

  if((option&0x01)) {
    lcd_mem_add = posy*1280 + posx*2;  // 1280 = 640 * 2

    for(row=0; row < height; row++) {
  	  temprow = row * width;
      for(col=0; col < width; col++) {
	    fbbackup[lcd_mem_add] = img[temprow + col];
	    lcd_mem_add++;
	  }
	  lcd_mem_add += (1280 - width);
    }
  }
}

// for 16bit mode
void lcd_putbmp(unsigned char *img, int posx, int posy, int width, int height, unsigned char option) {
  int row, col, temprow;
  ulong lcd_mem_add;

  width = width * 2;
  lcd_mem_add = posy*1280 + posx*2;  // 1280 = 640 * 2

  if((option&0x10) == 0x10) {
    if(frameflag == 0) {
	  frameflag = 1;
	  framebuf = framebuf0;
	  fbbackup = framebuf1;
    } else {
	  frameflag = 0;
	  framebuf = framebuf1;
	  fbbackup = framebuf0;
    }
  }

  for(row=height-1; row > 0; row--) {
	temprow = row * width;
    for(col=0; col < width; col++) {
	  framebuf[lcd_mem_add] = img[temprow + col];
	  lcd_mem_add++;
	}
	lcd_mem_add += (1280 - width);
  }

  LcdChangePage(framebuf);

  if((option&0x10) == 0x10)
    LcdChangePage(framebuf);

  if((option&0x01)) {
    lcd_mem_add = posy*1280 + posx*2;  // 1280 = 640 * 2

    for(row=height-1; row > 0; row--) {
	  temprow = row * width;
      for(col=0; col < width; col++) {
	    fbbackup[lcd_mem_add] = img[temprow + col];
	    lcd_mem_add++;
	  }
	  lcd_mem_add += (1280 - width);
    }
  }
}

//Ⱦ
void LcdPoint8(unsigned long point_x,unsigned long point_y,uchar temp)
{
  int start_point;
  volatile uchar color=temp;
  
//  printf("This is 8 color: %d point_x: %d point_y: %d \n",color,point_x,point_y);
  
  start_point=3838+point_x+640*point_y;
  LCD_BUF_CHAR(start_point) = color;
}
//Ⱦ
void LcdPoint16(unsigned long point_x,unsigned long point_y,ushort temp)
{
  int start_point;
  volatile ushort color=temp;
  //  point_x=point_x*2;
  
//  printf("This is 16 color: %d point_x: %d point_y: %d \n",color,point_x,point_y);
  
  start_point= 3836 + 2*point_x + 1280*point_y;
  LCD_BUF(start_point) = color;
}


int LcdModeApp(int depth)
{

  lcd_color_depth=depth;

  lcd_off();
  lcd_init(lcd_color_depth);
  lcd_on();
  //printf("%d Bit Graphic Mode\n",lcd_color_depth);
  return 0;
}

#if 0
int LcdMode(int argc, char * argv[])
{
  if(argc!=2)
    {
      printf("Usage : lcdmode [ 8|16 ]\n");
    }
  lcd_color_depth=aToi(argv[1]);

  lcd_off();
  lcd_init(lcd_color_depth);
  lcd_on();
  printf("%d Bit Graphic Mode\n",lcd_color_depth);
  return 0;
}

#endif

void SetLcdPalette(uchar palette_info[256][3])
{
  int i;
  ulong index=0;
  if(lcd_color_depth==LCD_COLOR_DEPTH8)
    {
      for (i = 0; i < 256; i ++){
	LCD_PAL(index) = (((palette_info[i][0] & 0xf8) << 8)         // Red
			  | ((palette_info[i][1] & 0xfc) << 3)                         // Green
			  | ((palette_info[i][2]) >> 3));                              // Blue
	
	index += 2;
      }	
        LCD_REG(LCD_LCCR0) = LCCR0_OUM          // Output FIFO Underrun Mask, 1= no interrupt
                                           | LCCR0_BM           // Branch Mask,1 = no interrupt
                                           | LCCR0_QDM          // Quick Disable Interrupt Mask, 1 = no interrupt
                                           | LCCR0_PAS          // Passive/Active Display select 1 = TFT
                                           | LCCR0_EFM          // End of FrameInterrupt Mask, 1 = no interrupt
                                           | LCCR0_IUM          // Input FIFO Underrun Mask, 1 = no interrupt
                                           | LCCR0_SFM          // Start of Frame Mask, 1 = no interrupt
                                           | LCCR0_LDM;         // LCD Disable Done Mask, 1 = no interrupt


//      printf("Success set palette!\n");
    }
//  else
//    {
//      printf("Do not required this function. This is 16bit color mode!\n");
//    }
}

void LcdChangePage(unsigned char *nPage)
{
        // kick off the frame descriptors
        // Setup the Palette Descriptor
//	LCD_START_BUF = LCD_BUF_ADD+3838 + (nPage*(512*1024));

        LCD_PDR(0x0) = LCD_FDR_ADD;                               // point to the frame buffer descriptor
        LCD_PDR(0x4) = LCD_PAL_ADD;                               // beginning of Palette Buffer in Memory
        LCD_PDR(0x8) = 0;                                         // Frame ID
        LCD_PDR(0xc) = BIT26 | 512;                               // PAL = 1, Frame Length in bytes = 512
                                                                                
        // Setup the Frame Descriptor
        LCD_FDR(0x0) = LCD_FDR_ADD;                               // point it to itself
        LCD_FDR(0x4) = nPage;//LCD_START_BUF;                               // beginning of Frame Buffer in Memory
                                                                                
        LCD_FDR(0x8) = 0;                                         // Frame ID
        LCD_FDR(0xc) = LCD_FB_SIZE(lcd_color_depth);              // Frame Length in bytes
                                                                                
                                                                                
        // For color depths of 4 or 8 we point the PXA250 to
        // the palette buffer descriptor, which will then point
        // to the frame buffer descriptor, while for 16 bpp
        // we point directly to the frame buffer descriptor,
        // thus skipping the palette buffer descriptor.
 //       LCD_FDR(0x0) = LCD_PDR_ADD;
        // point back to the palette descriptor
//        LCD_REG(LCD_FDADR0) = LCD_PDR_ADD;

//        LCD_REG(LCD_LCCR0) |= (LCD_REG(LCD_LCCR0) | LCCR0_ENB);
        // wait for power-up sequence to complete
//        udelay(10000);
}
