#ifndef _LCD_H
#define _LCD_H


//
// PXA255 LCD Controller Register and Bit Definitions
//
#include <drivers/lcd/types.h>


//---------------------------------------
// LCD Controller
#define LCD_BASE			0x44000000
#define LCD_REG(_x_)		*(vulong *)(LCD_BASE + _x_)	

//---------------------------------------
//     LCD Controller Register Offsets
#define LCD_LCCR0      			0x000          				// LCD controller control register 0
#define LCD_LCCR1      			0x004          				// LCD controller control register 1
#define LCD_LCCR2      			0x008          				// LCD controller control register 2
#define LCD_LCCR3      			0x00C          				// LCD controller control register 3
#define LCD_TRGBR      			0x040          				// TMED RGB Seed Register
#define LCD_TCR        			0x044          				// TMED Control Register
#define LCD_FDADR0     			0x200          				// DMA channel 0 frame descriptor address register
#define LCD_FSADR0     			0x204          				// DMA channel 0 frame source address register
#define LCD_FIDR0      			0x208          				// DMA channel 0 frame ID register
#define LCD_LDCMD0     			0x20C          				// DMA channel 0 command register
#define LCD_FDADR1     			0x210          				// DMA channel 1 frame descriptor address register
#define LCD_FSADR1     			0x214          				// DMA channel 1 frame source address register
#define LCD_FIDR1      			0x218          				// DMA channel 1 frame ID register
#define LCD_LDCMD1     			0x21C          				// DMA channel 1 command register
#define LCD_FBR0       			0x020          				// DMA channel 0 frame branch register
#define LCD_FBR1       			0x024          				// DMA channel 1 frame branch register
#define LCD_LCSR       			0x038          				// LCD controller status register
#define LCD_LIIDR      			0x03C          				// LCD controller interrupt ID register


// LCD controller control register 0
#define LCCR0_OUM 				BIT21						// Output FIFO Underrun Mask, 1 = no interrupt
#define LCCR0_BM 				BIT20						// Branch Mask
#define LCCR0_PDD(_x_)			((_x_ & 0xfff) << 12)		// Palette DMA Request Delay
#define LCCR0_QDM				BIT11						// Quick Disable Interrupt Mask, 1 = no interrupt
#define LCCR0_DIS				BIT10						// LCD Disable Status
#define LCCR0_DPD				BIT9						// Double-Pixel Data (DPD) pin mode (must be 0 for TFT)
#define LCCR0_PAS				BIT7						// Passive/Active Display select 1 = TFT
#define LCCR0_EFM				BIT6						// End of Frame Interrupt Mask, 1 = no interrupt
#define LCCR0_IUM				BIT5						// Input FIFO Underrun Mask, 1 = no interrupt
#define LCCR0_SFM				BIT4						// Start of Frame Mask, 1 = no interrupt
#define LCCR0_LDM				BIT3						// LCD Disable Done Mask, 1 = no interrupt
#define LCCR0_SDS				BIT2						// Single-/Dual-panel Display Select, 1 = Dual (must be 0 for TFT)
#define LCCR0_CMS				BIT1						// Color/Monochrome Select, 1 = Monochrome
#define LCCR0_ENB				BIT0						// LCD Controller Enable, 1 = Enable

// LCD controller control register 1
#define LCCR1_BLW(_x_)			((_x_ & 0xff) << 24)		// Beginning-of-Line pixel clock Wait count = BLW + 1
#define LCCR1_ELW(_x_)			((_x_ & 0xff) << 16)		// End-of-line pixel clock wait count = ELW + 1
#define LCCR1_HSW(_x_)			((_x_ & 0x3f) << 10)		// Horizontal sync pulse width = HSW + 1
#define LCCR1_PPL(_x_)			(((_x_) & 0x3ff) << 0)		// Pixels per line = PPL + 1

// LCD controller control register 2
#define LCCR2_BFW(_x_)			((_x_ & 0xff) << 24)		// Beginning-of-frame line clock wait count
#define LCCR2_EFW(_x_)			((_x_ & 0xff) << 16)		// End-of-frame line clock wait count
#define LCCR2_VSW(_x_)			((_x_ & 0x3f) << 10)		// Vertical sync pulse width = VSW + 1
#define LCCR2_LPP(_x_)			(((_x_) & 0x3ff) << 0)		// Lines per panel = LPP + 1

// LCD controller control register 3
#define LCCR3_DPC				BIT27						// Double Pixel Clock mode, 1 = LCLK = 2*PCD
#define LCCR3_BPP_1				(0x0 << 24) 				// 1-bits/pixel [4 entry, 8 byte palette buffer]
#define LCCR3_BPP_2				(0x1 << 24) 				// 2-bits/pixel [4 entry, 8 byte palette buffer]
#define LCCR3_BPP_4				(0x2 << 24) 				// 4-bits/pixel [16 entry, 32 byte palette buffer]
#define LCCR3_BPP_8				(0x3 << 24) 				// 8-bits/pixel [256 entry, 512 byte palette buffer]
#define LCCR3_BPP_16			(0x4 << 24)					// 16-bits/pixel [no palette buffer]
#define LCCR3_OEP				BIT23						// Output Enable Polarity, 0 = L_BIAS is Active High
#define LCCR3_PCP				BIT22						// Pixel Clock Polarity, 0 = LCD Data is Sampled on Rising Edge
#define LCCR3_HSP				BIT21						// Horizontal Sync Polarity, 0 = L_LCLK is Active High
#define LCCR3_VSP				BIT20						// Vertical Sync Polarity, L_FCLK is Active High
#define LCCR3_API(_x_)			((_x_ & 0xf) << 16)			// AC bias Pin transitions per Interrupt.
#define LCCR3_ACB(_x_)			((_x_ & 0xff) << 8)			// AC Bias pin frequency
#define LCCR3_PCD(_x_)			((_x_ & 0xff) << 0)			// Pixel Clock Divisor, Pixel Clock Frequency = LCLK/(2*(PCD+1)).

// LCD controller status register - These also clear the corresponding
// interrupt condition when a one is written to the bit
#define LCSR_SINT				BIT10						// Subsequent Interrupt status, maskable interrupt
#define LCSR_BS					BIT9						// Branch Status, maskable interrupt
#define LCSR_EOF				BIT8						// End Of Frame status, maskable interrupt
#define LCSR_QD					BIT7						// LCD Quick Disable status, maskable interrupt
#define LCSR_OU					BIT6						// Output FIFO Underrun status, maskable interrupt
#define LCSR_IUU				BIT5						// Input FIFO Underrun Upper panel status, maskable interrupt
#define LCSR_IUL				BIT4						// Input FIFO Underrun Lower panel status, maskable interrupt
#define LCSR_ABC				BIT3						// AC Bias Count status, nonmaskable interrupt
#define LCSR_BER				BIT2						// Bus error status, nonmaskable interrupt
#define LCSR_SOF				BIT1						// Start Of Frame status, maskable interrupt
#define LCSR_LDD				BIT0						// LCD Disable Done status, maskable interrupt

// 16-bit pixels are RGB 565 - LSB of RED and BLUE are tied low at the 
// LCD Interface, while the LSB of GREEN is loaded as 0



#if INCLUDE_CCD_CAMERA
	#define RED_SUBPIXEL(n)		(n & 0xf8) << 8
	#define GREEN_SUBPIXEL(n)	             (n & 0xfc) << 3
	#define BLUE_SUBPIXEL(n)	             (n & 0xf8) >>3
#else
	#define RED_SUBPIXEL(n)		(n & 0x1f) << 11
	#define GREEN_SUBPIXEL(n)	(n & 0x1f) << 6
	#define BLUE_SUBPIXEL(n)	(n & 0x1f) << 0
#endif


// define a simple VGA style 16-color pallette
#define	LU_BLACK		RED_SUBPIXEL(0x00) | GREEN_SUBPIXEL(0x00) | BLUE_SUBPIXEL(0x00)
#define	LU_BLUE			RED_SUBPIXEL(0x0f) | GREEN_SUBPIXEL(0x00) | BLUE_SUBPIXEL(0x00)
#define	LU_GREEN		RED_SUBPIXEL(0x00) | GREEN_SUBPIXEL(0x0f) | BLUE_SUBPIXEL(0x00)
#define	LU_CYAN			RED_SUBPIXEL(0x00) | GREEN_SUBPIXEL(0x0f) | BLUE_SUBPIXEL(0x0f)
#define	LU_RED			RED_SUBPIXEL(0x0f) | GREEN_SUBPIXEL(0x00) | BLUE_SUBPIXEL(0x00)
#define	LU_VIOLET		RED_SUBPIXEL(0x0f) | GREEN_SUBPIXEL(0x00) | BLUE_SUBPIXEL(0x0f)
#define	LU_YELLOW		RED_SUBPIXEL(0x0f) | GREEN_SUBPIXEL(0x0f) | BLUE_SUBPIXEL(0x00)
#define	LU_GREY			RED_SUBPIXEL(0x0f) | GREEN_SUBPIXEL(0x0f) | BLUE_SUBPIXEL(0x0f)
#define	LU_WHITE		RED_SUBPIXEL(0x17) | GREEN_SUBPIXEL(0x17) | BLUE_SUBPIXEL(0x17)
#define	LU_BRT_BLUE		RED_SUBPIXEL(0x00) | GREEN_SUBPIXEL(0x00) | BLUE_SUBPIXEL(0x1f)
#define	LU_BRT_GREEN	RED_SUBPIXEL(0x00) | GREEN_SUBPIXEL(0x1f) | BLUE_SUBPIXEL(0x00)
#define	LU_BRT_CYAN		RED_SUBPIXEL(0x00) | GREEN_SUBPIXEL(0x1f) | BLUE_SUBPIXEL(0x1f)
#define	LU_BRT_RED		RED_SUBPIXEL(0x1f) | GREEN_SUBPIXEL(0x00) | BLUE_SUBPIXEL(0x00)
#define	LU_BRT_VIOLET	RED_SUBPIXEL(0x1f) | GREEN_SUBPIXEL(0x00) | BLUE_SUBPIXEL(0x1f)
#define	LU_BRT_YELLOW	RED_SUBPIXEL(0x00) | GREEN_SUBPIXEL(0x1f) | BLUE_SUBPIXEL(0x1f)
#define	LU_BRT_WHITE	RED_SUBPIXEL(0x1f) | GREEN_SUBPIXEL(0x1f) | BLUE_SUBPIXEL(0x1f)

#ifndef PXA255_H
#define PXA255_H


//int lcd_init(int);
//void LcdPoint8(unsigned long,unsigned long,uchar);
//void LcdPoint16(unsigned long,unsigned long,ushort);

//void lcd_putchar(char c);


#endif //PXA255_H


//----------------------
#define LCD_BKL         17                      // GPIO 17 controls the LCD Backlight, 1 = On
#define LCD_COLOR_DEPTH4 4
#define LCD_COLOR_DEPTH8 8
#define LCD_COLOR_DEPTH16 16

//--------------------------------------------------------------------------
// The LCD frame buffer is fixed at 0xa2000000, which is 1Mbyte from the
// beginning of SDRAM space.  Note that we access it 16-bits at a time.
//


#define LCD_PDR_ADD             0xa3000000
#define LCD_PDR(_x_)    *(vulong *)(LCD_PDR_ADD + _x_)  // Palette Descriptor, Palette Buffer

#define LCD_FDR_ADD             0xa3000010
#define LCD_FDR(_x_)    *(vulong *)(LCD_FDR_ADD + _x_)  // Frame Descriptor, Frame Buffer

#define LCD_PAL_ADD             0xa3000020
#define LCD_PAL(_x_)    *(vushort *)(LCD_PAL_ADD + _x_) // Palette Buffer


#define LCD_BUF_ADD             0xa3000400
//#define LCD_BUF(_x_)    *(vushort *)(LCD_BUF_ADD + _x_) // Frame Buffer
// for elfos
#define LCD_BUF(_x_)    framebuf[_x_]

///By Hwang MK-------------------------------------------
#define LCD_BUF_CHAR(_x_)    *(vuchar *)(LCD_BUF_ADD+ _x_)
///-----------------------------------------------------

#define LCD_DOWNLOAD(_x_)	*(volatile ushort *)(APPRAMBASE_OVERRIDE + _x_) // image down load add

#define PC_CARD_BASE	0x48000000
#define PC_CARD_REG(_x_)		*(vulong *)(PC_CARD_BASE + _x_)

#define TFT_LCD_640X480 1
//--------------------------------------------------------------------------
// function prototypes and externs
//

void lcd_on(void);
void lcd_off(void);
void lcd_putchar(char c);
void lcd_putstring(char *s);
void lcd_puthex(const unsigned long h);
void lcd_setpos(int col, int row);
void lcd_setfgbg(int fg, int bg);
void lcd_fillbg(int color);
void lcd_putpic(unsigned char *img, int posx, int posy, int width, int height, unsigned char option);
void lcd_putbmp(unsigned char *img, int posx, int posy, int width, int height, unsigned char option);
void lcd_writechar(uchar c);
void lcd_clr_row(int char_row);
int lcd_init(int);

//application library
int LcdModeApp(int);
int LcdMode(int,char **);
void LcdPoint(unsigned long,unsigned long,uchar);
void LcdPoint16(unsigned long,unsigned long,ushort);
void SetLcdPalette(uchar [][]);

#endif //_LCD_H
