
#include <arch/arch.h>
#include <drivers/rtc/rtc.h>
#include <drivers/serial/serial.h>
#include <drivers/fnd/fnd.h>


int rtc_mode = RTC_TIME;

unsigned int rtc_fnd[6];
unsigned int rtc_alarm_fnd[6];


void rtc_display(void);
void rtc_alarm_display(void);


int rtc_status(int mode)
{
	if (mode != RTC_CHECK)
		rtc_mode = mode;

	return rtc_mode;
}


void rtc_init(int hour, int minute, int second)
{
//----------------- Real Time Clock interrupt setting
	// Interrupt Controller Level Register set IRQ
	INT_REG(INT_ICLR) &= ~(BIT30);

	// Interrupt Controller Mask Register set Masked
	INT_REG(INT_ICMR) |= (BIT30);

//----------------- Real Time Clock setting
	RTC_REG(RTC_RTTR) = (RTC_LCK | RTC_1HZ);

	RTC_REG(RTC_RTSR) = (RTC_HZE | RTC_HZ);

	RTC_REG(RTC_RCNR) = 
		(3600 * hour) + (60 * minute) + second;
}

void rtc_handler(void)
{
	if (rtc_mode == RTC_ALARM)
		rtc_alarm_display();
	else
		rtc_display();

	RTC_REG(RTC_RTSR) |= (RTC_HZE | RTC_HZ);
}

void rtc_set(int hour, int minute, int second)
{
	RTC_REG(RTC_RCNR) =
		(3600 * hour) + (60 * minute) + second;
}

unsigned long rtc_get(void)
{
	unsigned int rtc_hour, rtc_minute, rtc_second, rtc_temp;
	unsigned long rtc_time = RTC_REG(RTC_RCNR);

	rtc_hour = rtc_time / 3600;
	rtc_temp = rtc_time % 3600;
	rtc_minute = rtc_temp / 60;
	rtc_temp = rtc_temp % 60;
	rtc_second = rtc_temp;

	rtc_fnd[0] = rtc_hour / 10;
	rtc_fnd[1] = rtc_hour % 10;
	rtc_fnd[2] = rtc_minute / 10;
	rtc_fnd[3] = rtc_minute % 10;
	rtc_fnd[4] = rtc_second / 10;
	rtc_fnd[5] = rtc_second % 10;

	return rtc_time;
}

void rtc_display(void)
{
	int16u fnd_time[5];

	rtc_get();
	fnd_time[0] = fnd_segment(rtc_fnd[0], 0, 0);
	fnd_time[1] = fnd_segment(rtc_fnd[1], 1, 1);
	fnd_time[2] = fnd_segment(rtc_fnd[2], 0, 2);
	fnd_time[3] = fnd_segment(rtc_fnd[3], 1, 3);
	fnd_time[4] = fnd_segment(rtc_fnd[4], 0, 4);
	fnd_time[5] = fnd_segment(rtc_fnd[5], 1, 5);
	fnd_display(fnd_time);
}



void rtc_alarm_init(int hour, int minute, int second)
{
//----------------- Real Time Clock interrupt setting
	// Interrupt Controller Level Register set IRQ
	INT_REG(INT_ICLR) &= ~(BIT31);

	// Interrupt Controller Mask Register set Masked
	INT_REG(INT_ICMR) |= (BIT31);

//----------------- Real Time Clock Alarm set
	RTC_REG(RTC_RTSR) |= (RTC_ALE | RTC_AL);

	// RTC Alarm Register set
	RTC_REG(RTC_RTAR) = rtc_get() +
			(3600 * hour) + (60 * minute) + second;

	rtc_status(RTC_ALARM);
}

void rtc_alarm_release(void)
{
	rtc_status(RTC_TIME);
}

void rtc_alarm_handler(void)
{
	SerialOutputString("have a good time with warm coffee!\n");
	rtc_alarm_release();
}

unsigned long rtc_alarm_remain(void)
{
	unsigned int rtc_hour, rtc_minute, rtc_second, rtc_temp;
	unsigned long rtc_time, rtc_alarm, rtc_remind;

	rtc_alarm = RTC_REG(RTC_RTAR);
	rtc_time = RTC_REG(RTC_RCNR);
	rtc_remind = rtc_alarm - rtc_time;

	rtc_hour = rtc_remind / 3600;
	rtc_temp = rtc_remind % 3600;
	rtc_minute = rtc_temp / 60;
	rtc_temp = rtc_temp % 60;
	rtc_second = rtc_temp;

	rtc_alarm_fnd[0] = rtc_hour / 10;
	rtc_alarm_fnd[1] = rtc_hour % 10;
	rtc_alarm_fnd[2] = rtc_minute / 10;
	rtc_alarm_fnd[3] = rtc_minute % 10;
	rtc_alarm_fnd[4] = rtc_second / 10;
	rtc_alarm_fnd[5] = rtc_second % 10;

	return rtc_remind;
}

void rtc_alarm_display(void)
{
	int16u fnd_time[5];
	unsigned long remain = rtc_alarm_remain();

	if (remain == 0) {
		fnd_time[0] = fnd_segment(0xc, 0, 0);
		fnd_time[1] = fnd_segment(0x0, 1, 1);
		fnd_time[2] = fnd_segment(0xf, 0, 2);
		fnd_time[3] = fnd_segment(0xf, 1, 3);
		fnd_time[4] = fnd_segment(0xe, 0, 4);
		fnd_time[5] = fnd_segment(0xe, 1, 5);
		rtc_alarm_handler();
	}
	else {
		fnd_time[0] = fnd_segment(rtc_alarm_fnd[0], 0, 0);
		fnd_time[1] = fnd_segment(rtc_alarm_fnd[1], 1, 1);
		fnd_time[2] = fnd_segment(rtc_alarm_fnd[2], 0, 2);
		fnd_time[3] = fnd_segment(rtc_alarm_fnd[3], 1, 3);
		fnd_time[4] = fnd_segment(rtc_alarm_fnd[4], 0, 4);
		fnd_time[5] = fnd_segment(rtc_alarm_fnd[5], 1, 5);
	}
	fnd_display(fnd_time);
}

