/***************************************************************************** 
 * File        : mm.c
 * Date        : 2007/04/04
 * Author      : Sun Joon Gyu
 * Modified    : 2007/04/04
 * Description : Rutines for memory management
 * ***************************************************************************/
#ifndef MM_TEST
#include<kernel/mm.h>
#include<kernel/thread.h>
#include<kernel/schedule.h>
#include<kernel/list.h>
#include<drivers/serial/serial.h>


#else
#include"mm.h"
#include"types.h"
#include"list.h"
memclt_t *pThreadMemList_g;	/* a first block for heap spaces */
#endif //MM_TEST


memclt_t *pHeapListHead_g;	/* a head for memory controller list*/
memclt_t *pFirstHeapHead_g;	/* a first block for heap spaces */



memclt_t *pHeapListTail_g;	/* It indicates the end of heap */

int8s __attribute__((aligned(256))) heap[1024*1024];		/* only for testing this rutines */




/******************************************************************
 * Function : Kern_MemInit()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Operation : It simply initialize memory control block for heap.
 *             At first, it craetes a header controlling memory spaces.
 *             And it puts the header into the top of the heap.
 *****************************************************************/

bool Kern_MemInit()
{
	
	SerialOutputString("Memory Init Started ...\n");
	pHeapListHead_g = (memclt_t *)START_HEAP_ADDR;
	//pHeapListTail_g = ((memclt_t *)END_HEAP_ADDR )-1;
	INIT_LIST_HEAD(&pHeapListHead_g->memLink);
	INIT_LIST_HEAD(&pHeapListHead_g->allocLink);
	INIT_LIST_HEAD(&pHeapListHead_g->freeMemLink);
	SerialOutputString("pHeapListHead_g Initialized...\n");

	pHeapListHead_g->status = INVALID;

	memclt_t *memHeader = pFirstHeapHead_g = 
			(memclt_t *)(((int8u *)pHeapListHead_g) + BLOCK_SIZE);	/*Next block */
	/* Initializing a header of heap*/
	INIT_LIST_HEAD(&memHeader->memLink);
	INIT_LIST_HEAD(&memHeader->allocLink);
	INIT_LIST_HEAD(&memHeader->freeMemLink);
	SerialOutputString("memHeader Initialized...\n");

	memHeader->status = AVAILABLE;

	/* should subtract the size of one memory block and a list header*/
	memHeader->nblocks = (int32u) (((int8s *)END_HEAP_ADDR - 
							(int8s *)pFirstHeapHead_g) >> BLOCK_SIZE_SHIFT) - 1; 	



	if(memHeader->nblocks <= 0)
		return FALSE;

	ListAdd(&memHeader->freeMemLink, &pHeapListHead_g->freeMemLink);
	ListAdd(&memHeader->memLink, &pHeapListHead_g->memLink);
	SerialOutputString("List Add Finished ...\n");

#ifdef MM_TEST
	pThreadMemList_g = malloc(sizeof(memclt_t));
	INIT_LIST_HEAD(&pThreadMemList_g->memLink);
	INIT_LIST_HEAD(&pThreadMemList_g->allocLink);
#endif
	return TRUE;

}



/***********************************************************************************************
 * Function : Kern_WantedBlkFind(size_t size)
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : size_t size - the number of bytes desired
 * Return type : memclt_t *- it returns an address indicating specific memory which is available.
 * Operation : Sequencially, It goes around memory control blocks until it finds a memory control 
 *             block which can be allowed to put desired size of bytes
 ***********************************************************************************************/


static memclt_t * 
Kern_WantedBlkFind(size_t size)
{
	int32u nblocks = 0;

//CRITICAL_SECTION_ENTER();
	memclt_t * pHead = (memclt_t *)pHeapListHead_g;

	size += (BLOCK_SIZE-1);
	nblocks = size >> BLOCK_SIZE_SHIFT;

	memclt_t *iter;
	ListForEachEntry(iter,&pHead->freeMemLink,freeMemLink){
	
			if(iter->nblocks >= nblocks){
					//CRITICAL_SECTION_LEAVE();
					return iter;
			}

	}
//CRITICAL_SECTION_LEAVE();

	return NULL;
		
}





/*************************************************************************************************
 * Function : Kern_MemBlkGet()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : memclt_t * pBlkAddr - the address picked in the function Kern_WantedBlkFind() 
 *             size_t size - the number of bytes desired
 * Return type : int8s - It returns a condition value indicating if this rutines completed successfully or not
 * Operation : It adjust neibor's links and create new memory control block to control extra memory spaces
 * ************************************************************************************************/


static int8s 
Kern_MemBlkGet(memclt_t * pBlkAddr,size_t size)
{

	int32u nblocks;
	size += (BLOCK_SIZE-1);
	nblocks = size >> BLOCK_SIZE_SHIFT;
	SerialOutputString("pBlkAddr : ");
	SerialOutputHex(pBlkAddr);
	SerialOutputString("\nnblocks : ");
	SerialOutputHex(nblocks);
	
//CRITICAL_SECTION_ENTER();
	/* Just in case there is something wrong with memory block pointer */
	if((pBlkAddr->nblocks) == nblocks){

		pBlkAddr->status = ALLOCATED;
#ifndef MM_TEST
		ListDel(&pBlkAddr->freeMemLink);
		ListAdd(&pBlkAddr->allocLink, &CurrentThread()->memList.allocLink);
#else
		ListDel(&pBlkAddr->freeMemLink);
		ListAdd(&pBlkAddr->allocLink, &pThreadMemList_g->allocLink);
#endif
//CRITICAL_SECTION_LEAVE();
	}
	else if((pBlkAddr->nblocks) > nblocks){
			SerialOutputString("\npBlkAddr->nblocks : ");
			SerialOutputHex(pBlkAddr->nblocks);
		pBlkAddr->status = ALLOCATED;
//CRITICAL_SECTION_LEAVE();
		size_t extraBlkSize = pBlkAddr->nblocks - nblocks - 1;  
			/* should subtract an additional memory control header */

		memclt_t * pExtraMemBlk = (memclt_t *)((int8s *)pBlkAddr
				+ ((nblocks + 1) << BLOCK_SIZE_SHIFT)); 
		INIT_LIST_HEAD(&pExtraMemBlk->memLink);
		INIT_LIST_HEAD(&pExtraMemBlk->allocLink);
		INIT_LIST_HEAD(&pExtraMemBlk->freeMemLink);
			SerialOutputString("\n pExtraMemBlk Initialized  \n");
		
		pExtraMemBlk->nblocks = extraBlkSize;
		pExtraMemBlk->status = AVAILABLE;

		pBlkAddr->nblocks = nblocks;
		pBlkAddr->status = ALLOCATED;

		ListAdd(&pExtraMemBlk->memLink,&pBlkAddr->memLink);
		ListAdd(&pExtraMemBlk->freeMemLink,&pBlkAddr->freeMemLink);
		ListDel(&pBlkAddr->freeMemLink);

			SerialOutputString("\n List Operation Finished\n");
#ifndef MM_TEST
			SerialOutputString("\n List Add to Thread Operation started\n");
		ListAdd(&pBlkAddr->allocLink, &CurrentThread()->memList.allocLink);
			SerialOutputString("\n List Add to Thread Operation Finished\n");
#else
		ListAdd(&pBlkAddr->allocLink, &pThreadMemList_g->allocLink);
#endif

	}
	else{

//CRITICAL_SECTION_LEAVE();
		return (int8s)FALSE;
	}


	return TRUE;	

}	




#if 0

/*****************************************************************************
 * Function : Kern_BlkTblRegister()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : int32u *pReqVar - a pointer variable which is asking the 
 * 					bytes of memory spaces
 *             memclt_t * - A address pointing at the location which is just 
 *             		    allocated
 *
 * Return type : int8s - It returns a condition value indicating if this rutines 
 * 			completed successfully of not
 * Operation : It registers a address of a desired pointer variable into memory 
 * 		control block and also put the control block into the list in 
 * 		thread control block
 ******************************************************************************/

int8s 
Kern_BlkTblRegister(memclt_t * pBlkAddr)
{
CRITICAL_SECTION_ENTER();
	if(pCurrThread_g!= NULL){
		if(pCurrThread_g->pMemList != NULL){
			pBlkAddr->pAllocNext = pCurrThread_g->pMemList;
			pCurrThread_g->pMemList = pBlkAddr;
		}else{
			pBlkAddr->pAllocNext = NULL;
			pCurrThread_g->pMemList = pBlkAddr;

		}
	}

	pBlkAddr->pVar = pReqVar;
CRITICAL_SECTION_LEAVE();
	return TRUE;
}


#endif


/******************************************************************************
 * Function :  MemRelease()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : void *p - the address of memory wanted to be released
 * Return type : int8s - It returns a condition value indicating if this rutines 
 			completed successfully of not
 * Operation : it simply makes the bunch of memory blocks owned 
 		by the thread free. 
 ******************************************************************************/

int8s MemRelease(void *p)
{

	bool found = FALSE;

	memclt_t *release = (memclt_t *) 
			(( ((int32u)p) - (BLOCK_SIZE - 1)) >> BLOCK_SIZE_SHIFT);
	release = (memclt_t *) (((int32u)release) << BLOCK_SIZE_SHIFT);
#ifdef MM_TEST
	memclt_t * mem_head = pThreadMemList_g;
#else
	memclt_t * mem_head = &CurrentThread()->memList;
#endif

	memclt_t *iter;

	ListForEachEntry(iter,&mem_head->allocLink,allocLink){
			if(iter == release){
					found=1;
					ListDel(&release->allocLink);
					break;
			}
	}


/*CRITICAL_SECTION_ENTER() */

	if(found){
			memclt_t *prev_mem = ListEntry(release->memLink.prev, memclt_t, memLink);
			memclt_t *next_mem = ListEntry(release->memLink.next, memclt_t, memLink);



			if(prev_mem->status == AVAILABLE){
					Kern_MemMerge(prev_mem, release , BACKWARD);

					if(next_mem->status == AVAILABLE){
							Kern_MemMerge(prev_mem, next_mem, FORWARD);
					}
			}else if(next_mem->status == AVAILABLE){
							Kern_MemMerge(release, next_mem, FORWARD);
			}else{
					release->status = AVAILABLE;
					memclt_t *iter;
					ListForEachEntry(iter, &release->memLink, memLink){
							ListAddTail(&release->freeMemLink,&iter->freeMemLink);
							break;

					}

			}

			/*CRITICAL_SECTION_EXIT() */

			return TRUE;
	} else
			return FALSE;
	
}



/*************************************************************************************************
 * Function :  Kern_MemMerge()
 * Date	: 2007/04/10
 * Written by : Sun Joon Gyu
 * Arguments : memclt_t * pPrevBlk - 
 * Return type : int8s - It returns a condition value indicating if this rutines completed successfully of not
 * Operation : it simply makes the bunch of memory blocks owned by the thread free. 
 ***************************************************************************************************/
#if 1
static bool 
Kern_MemMerge(memclt_t * pPrevBlk, memclt_t * pNextBlk,int8u dir)
{

	int32u mergedSize = (pPrevBlk->nblocks + pNextBlk->nblocks) + 1;	/* include one block used as a header of memory blocks */

	struct list_head *next_blk_list_free = &pNextBlk->freeMemLink;
	struct list_head *next_blk_list_mem = &pNextBlk->memLink;

	struct list_head *prev_blk_list_free = &pPrevBlk->freeMemLink;
	struct list_head *prev_blk_list_mem = &pPrevBlk->memLink;

	next_blk_list_free->next->prev = prev_blk_list_free;
	prev_blk_list_free->next = next_blk_list_free->next;
	if(dir == BACKWARD){
			memclt_t *pprev = ListEntry(prev_blk_list_mem->prev, memclt_t, memLink);
			ListAdd(prev_blk_list_free, &pprev->freeMemLink);
	}
	ListDel(next_blk_list_mem);
	//ListAdd(next_blk_list_free, prev_blk_list_free);
	pPrevBlk->nblocks= mergedSize;
	pPrevBlk->status = AVAILABLE;

#ifndef MM_TEST
		pPrevBlk->threadID = CurrentThread()->tid;	
#endif

	return TRUE;	

}
#endif 

int32u *MemAlloc(size_t size)
{
		memclt_t *pBlkAddr;

	if( (pBlkAddr = Kern_WantedBlkFind(size)) == NULL){
			return NULL;
		/* The thread tried to get memory from heap area should be blocked!! */
		/* DbgPrint("Not enough memory in Heap area\n"); */
	}
	SerialOutputString("No problem with Kern_WantedBlkFind\n");
	if(Kern_MemBlkGet(pBlkAddr, size) == FALSE){
			return NULL;
		/* DbgPrint("Something wrong!!\n"); */
	}
	SerialOutputString("No problem with Kern_MemBlkGet\n");

	return (int32u *)((int8s *)pBlkAddr + BLOCK_SIZE);	/* return the address except memory header */
}


#ifdef MM_TEST
int32s 
main()
{
	bool k = Kern_MemInit();

	int *a1 = MemAlloc(10);
	int *a2 = MemAlloc(100);
	int *a3 = MemAlloc(1000);
	int *a4 = MemAlloc(10000);
	int *a5 = MemAlloc(100000);
	int *a7 = MemAlloc(100000);
	int *a8 = MemAlloc(100000);
	int *a9 = MemAlloc(100000);
	int *a10 = MemAlloc(100000);

	memclt_t *iter;
	MemRelease(a4);
	MemRelease(a2);
	MemRelease(a1);
	MemRelease(a3);
	MemRelease(a5);



	int *b1 = MemAlloc(30);
	int *b2 = MemAlloc(2048);
	MemRelease(b1);
	b1 = MemAlloc(10000);
	MemRelease(b2);
	MemRelease(b1);


	
	ListForEachEntry(iter, &pHeapListHead_g->memLink, memLink){
			printf("memLink\n");

			printf("iter nblocks : %d \n", iter->nblocks);
	}
	ListForEachEntry(iter, &pHeapListHead_g->allocLink, allocLink){
			printf("allocLink\n");
			printf("iter nblocks : %d \n", iter->nblocks);
	}
#if 1
	ListForEachEntry(iter, &pHeapListHead_g->freeMemLink, freeMemLink){
			printf("freeMemLink\n");
			printf("iter nblocks : %d \n", iter->nblocks);
	}
#endif 


}

#endif //MM_TEST
