/*************************************************************************** 
 * File        : pipe.c
 * Date        : 2007/04/20
 * Author      : Sun Joon Gyu
 * Modified    : 2007/04/20
 * Description : 
 * ***************************************************************************/
#include<kernel/pipe.h>
#include<kernel/schedule.h>
#define GetPipeOp()	(&pipeOp_g) 

const pipe_op_t pipeOp_g = {
	.Create = CreatePipe,
	.Open = OpenPipe,
	.Read = ReadFromPipe,
	.Write = WriteToPipe,
	.Close = ClosePipe,
	.Destroy = DestroyPipe
};


/*A table for managing pipes */
pipe_t *pipeTable_g[MAX_PIPE];







	
/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/
int8s 
Kern_InitPipeOp()
{
#if 0
	Kern_RegisterDev(1,NULL,OpenPipe,ReadFromPipe,WriteToPipe,ClosePipe,NULL,DestroyPipe);
#endif

	return TRUE;
}







/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8u
CreatePipe(int8u pid, size_t size)
{
	pipe_t *pipe;
	int32s err;

CRITICAL_SECTION_ENTER();
	if(pipeTable_g[pid] != NULL){
		return -1;
	}
	pipeTable_g[pid] = (pipe_t *)1;		// mark this entry which is going to be allocated 
					// just in case other thread steal it
CRITICAL_SECTION_LEAVE();


	pipe = (pipe_t *)MemAlloc(sizeof(struct pipe));
#if 0
	pipe->p_op = Kern_GetDevOp(1,(int8s *)&err);
#endif
	pipe->p_op = GetPipeOp();

	pipe->data = (pipe_t *)MemAlloc(size);
	pipe->in = pipe->out = 0;
	pipe->count = 0;
	pipe->buff_size = size;
	pipe->owner = (void *)Kern_GetCurrThread();
	pipe->pid = pid;

CRITICAL_SECTION_ENTER();
	pipeTable_g[pid] = pipe;
CRITICAL_SECTION_LEAVE();

	return TRUE; 

}








/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

pipe_t *
OpenPipe(int8u pid, int8u mode)
{
	pipe_t *pipe;

CRITICAL_SECTION_ENTER();
	if(pipeTable_g[pid] == NULL){
	
CRITICAL_SECTION_LEAVE();
		return NULL;
	}


	pipe = pipeTable_g[pid];
	pipe->count++;
CRITICAL_SECTION_LEAVE();

	return pipe;

}







/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s *
ReadFromPipe(pipe_t *pipe, int8s *dest, size_t size)
{
	
	int8s *src;
	int32u i=0;
	if(pipe == NULL && dest == NULL){
		return 0;
	}
	
	src = (int8s *)pipe->data;

	while(i < size){
		if(pipe->in == pipe->out)
//			WaitForWriter();
		;

//		dest[i] = src[pipe->out];
		pipe->out = (pipe->out + 1 ) % pipe->buff_size;
		++i;

//		WakeUpReader();
	}

	return dest;
}







/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

size_t
WriteToPipe(pipe_t *pipe, const int8s *src, size_t size )
{
	int8s *dest;
	int32u i=0;
	if(pipe == NULL && dest == NULL){
		return 0;
	}

	dest = (int8s *)pipe->data;
	while(i < size){

//ENTER_CRITICAL_SECTION
		if(pipe->in+1  == pipe->out)	// then buffer is full
//			WaitForReader();
		;

		dest[pipe->in] = src[i];
		pipe->in = (pipe->in + 1 ) % pipe->buff_size;
		++i;
//EXIT_CRITICAL_SECTION
//		WakeUpWriter(pipe->);
		
	}

	return i;

}






/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s
ClosePipe(pipe_t *pipe)
{
CRITICAL_SECTION_ENTER();
	pipe->count--;
CRITICAL_SECTION_LEAVE();

}







/*************************************************************************** 
 * Function    : 
 * Date        : 
 * Author      : 
 * Parameter   : 
 * Return value:
 * Description : 
 * ***************************************************************************/

int8s
DestroyPipe(pipe_t *pipe)
{

	if(pipe==NULL){
		return 1;
	}else{

CRITICAL_SECTION_ENTER();
		if(pipe->count-1 != 0){
CRITICAL_SECTION_LEAVE();
			return FALSE;/*The count should be zero to destroy a pipe */
		}else{
			MemRelease(pipe->data);
			MemRelease(pipe);
		}
CRITICAL_SECTION_LEAVE();
	}
#if 0
	Kern_DeallocDev(1);		
#endif
}
