/*************************************************************************** 
 * File        : pipe.h
 * Date        : 2007/04/20
 * Author      : Sun Joon Gyu
 * Modified    : 2007/04/20
 * Description : 
 * ***************************************************************************/
#ifndef _PIPE_H
#define _PIPE_H

#include <arch/types.h>
#include <kernel/iosub.h>
#include <kernel/thread.h>
#include <arch/port.h>

#define MAX_PIPE	255

#define NONBLOCKING	0x01
#define BLOCKING	0x00

typedef struct pipe_operation {
	int32s (*Create)();
	int32s (*Open)();
	int32s (*Read)();
	int32s (*Write)();
	int32s (*Close)();
	int32s (*Ioctl)();
	int32s (*Destroy)();
}pipe_op_t;


typedef struct pipe {
	int16u pid;
	int16u count;	// The number of threads using this pipe
	pipe_op_t *p_op;		// Operator for pipe
	int16u in, out;	// Index of entrance and exit of the buffer
	size_t buff_size;
	void *data;
	void *owner;		// Threads which creates this pipe
} pipe_t;



int8s InitPipeOp();
int8u CreatePipe(int8u pid, size_t size);
pipe_t *OpenPipe(int8u pid, int8u mode);
int8s *ReadFromPipe(pipe_t *pipe, int8s *dest, size_t size);
size_t WriteToPipe(pipe_t *pipe, const int8s *src, size_t size );
int8s ClosePipe(pipe_t *pipe);
int8s DestroyPipe();



#endif // _PIPE_H
